/***************************************************************************
                          ccharacterprofile.cpp
                           -------------------
    description          : Character Profile Class
    begin                : Sun Oct 17 1999
    copyright            : (C) 1999 by Stephan Uhlmann
    email                : suhlmann@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ccharacterprofile.h"

CCharacterProfile::CCharacterProfile(){
 filename="";
}
CCharacterProfile::~CCharacterProfile(){
}


QString CCharacterProfile::getFileName()
{
 return filename;
}

void CCharacterProfile::setFileName(QString f)
{
 filename=f;
}

void CCharacterProfile::readData()
{
    KSimpleConfig mudconf(QDir::homeDirPath()+"/.kde/share/apps/kmud/profiles/"+filename);

    readAliases();
    readTriggers();
}

void CCharacterProfile::readAliases()
{
    AliasEntry* alias;
    QString groupname, n, d, m, tmp;
    KSimpleConfig aliasdb(QDir::homeDirPath()+"/.kde/share/apps/kmud/profiles/"+filename);
    KGroupIterator *gi;
    uint i = 0;

    aliases.setAutoDelete(true);
    aliases.clear();

    gi = aliasdb.groupIterator();
    gi->toFirst();

    while (gi->current())
    {
        groupname = gi->currentKey();

        if (groupname == "Aliases")
        {
            QString name = "";
            aliasdb.setGroup("Aliases");

            while (name != NULL) {
                alias = new AliasEntry;
                n.setNum(i).append("Name");
                d.setNum(i).append("Def");
                m.setNum(i).append("Macro");

                tmp = aliasdb.readEntry(n, "");
                if (tmp == "")
                    break;
                else
                    alias->name = tmp;
                alias->definition = aliasdb.readEntry(d, "");
                alias->macro = aliasdb.readNumEntry(m, 0);
                aliases.append(alias);
                i++;
            }
        }
        ++(*gi);
    }
    delete gi;
}

void CCharacterProfile::readTriggers()
{
    TriggerEntry* trigger;
    QString groupname, n, d;
    KSimpleConfig triggerdb(QDir::homeDirPath()+"/.kde/share/apps/kmud/profiles/"+filename);
    KGroupIterator *gi;
    uint i = 0;

    triggers.setAutoDelete(true);
    triggers.clear();

    gi = triggerdb.groupIterator();
    gi->toFirst();

    while (gi->current())
    {
        groupname = gi->currentKey();

        if (groupname == "Triggers")
        {
            QString name = "";
            triggerdb.setGroup("Triggers");

            while (name != NULL) {
                trigger = new TriggerEntry;
                n.setNum(i).append("Name");
                d.setNum(i).append("Def");
                name = triggerdb.readEntry(n, NULL);
                if (name == NULL)
                    break;
                else
                    trigger->name = name;
                trigger->definition = triggerdb.readEntry(d, "");
                triggers.append(trigger);
		i++;
            }
        }
        ++(*gi);
    }
    delete gi;
}

void CCharacterProfile::writeData()
{
    KSimpleConfig mudconf(QDir::homeDirPath()+"/.kde/share/apps/kmud/profiles/"+filename);

    writeAliases();
    writeTriggers();
}

void CCharacterProfile::writeAliases()
{
    KSimpleConfig aliasdb(QDir::homeDirPath()+"/.kde/share/apps/kmud/profiles/"+filename);
    KGroupIterator* gi;
    QString groupname, n, d, m;
    uint i;

    gi = aliasdb.groupIterator();
    gi->toFirst();

    while (gi->current()) {
        groupname = gi->currentKey();
        if (groupname = "Aliases")
            aliasdb.deleteGroup(groupname);
        ++(*gi);
    }
    delete gi;

    aliasdb.sync();

    /** Write the aliases */
    aliasdb.setGroup("Aliases");
    for (i = 0; (unsigned int)i < aliases.count(); i++) {
        n.setNum(i).append("Name");
        d.setNum(i).append("Def");
        m.setNum(i).append("Macro");
        aliasdb.writeEntry(n, aliases.at(i)->name);
        aliasdb.writeEntry(d, aliases.at(i)->definition);
        aliasdb.writeEntry(m, aliases.at(i)->macro);
    }
}

void CCharacterProfile::writeTriggers()
{
    KSimpleConfig triggerdb(QDir::homeDirPath()+"/.kde/share/apps/kmud/profiles/"+filename);
    KGroupIterator* gi;
    QString groupname, n, d;
    uint i;

    gi = triggerdb.groupIterator();
    gi->toFirst();

    while (gi->current()) {
        groupname = gi->currentKey();
        if (groupname = "Triggers")
            triggerdb.deleteGroup(groupname);
        ++(*gi);
    }
    delete gi;

    triggerdb.sync();

    /** Write the triggers */
    triggerdb.setGroup("Triggers");
    for (i = 0; (unsigned int)i < triggers.count(); i++) {
        n.setNum(i).append("Name");
        d.setNum(i).append("Def");
        triggerdb.writeEntry(n, triggers.at(i)->name);
        triggerdb.writeEntry(d, triggers.at(i)->definition);
    }
}

void CCharacterProfile::insertAlias(QString name, QString def, int mac) {
    AliasEntry* alias;
    int i = 0;

    alias = firstAlias();
    while ((alias != NULL) && (alias->name != name)) {
        alias = nextAlias();
        i++;
    }

    alias = new AliasEntry;
    alias->name = name;
    alias->definition = def;
    alias->macro = mac;

    aliases.append(alias);
}

void CCharacterProfile::insertTrigger(QString name, QString def) {
    TriggerEntry* trigger;

    trigger = firstTrigger();
    while ((trigger != NULL) && (trigger->name != name)) {
        trigger = nextTrigger();
    }

    trigger = new TriggerEntry;
    trigger->name = name;
    trigger->definition = def;

    triggers.append(trigger);
}

void CCharacterProfile::removeAlias(QString name) {
    AliasEntry* alias;

    alias = aliases.first();
    while (alias != NULL) {
        if (alias->name == name) {
            aliases.remove(alias);
        }
        alias = aliases.next();
    }
}

void CCharacterProfile::removeTrigger(QString name) {
    TriggerEntry* trigger = new TriggerEntry;

    trigger = triggers.first();
    while (trigger != NULL) {
        if (trigger->name == name) {
            triggers.remove(trigger);
        }
        trigger = triggers.next();
    }
}

AliasEntry* CCharacterProfile::getAlias(QString name)
{
    AliasEntry* alias;

    alias = firstAlias();
    while (alias != NULL) {
        if (alias->name == name)
            return alias;
        alias = nextAlias();
    }
    return NULL;
}

TriggerEntry* CCharacterProfile::getTrigger(QString n)
{
    TriggerEntry* trigger = new TriggerEntry;

    trigger = firstTrigger();
    while (trigger != NULL) {
        if (trigger->name == n) break;
        trigger = nextTrigger();
    }
    return trigger;
}

AliasEntry* CCharacterProfile::firstAlias() {
    currentAliasIndex = 0;
    if (aliases.count() > (unsigned int)currentAliasIndex) return aliases.at(currentAliasIndex);
    else return NULL;
}

AliasEntry* CCharacterProfile::nextAlias() {
    currentAliasIndex++;
    if (aliases.count() > (unsigned int)currentAliasIndex) return aliases.at(currentAliasIndex);
    else return NULL;
}

AliasEntry* CCharacterProfile::currentAlias() {
    if (aliases.count() > (unsigned int)currentAliasIndex) return aliases.at(currentAliasIndex);
    else return NULL;
}




TriggerEntry* CCharacterProfile::firstTrigger() {
    currentTriggerIndex = 0;
    if (triggers.count() > (unsigned int)currentTriggerIndex) return triggers.at(currentTriggerIndex);
    else return NULL;
}

TriggerEntry* CCharacterProfile::nextTrigger() {
    currentTriggerIndex++;
    if (triggers.count() > (unsigned int)currentTriggerIndex) return triggers.at(currentTriggerIndex);
    else return NULL;
}

TriggerEntry* CCharacterProfile::currentTrigger() {
    if (triggers.count() > (unsigned int)currentTriggerIndex) return triggers.at(currentTriggerIndex);
    else return NULL;
}
