/***************************************************************************
                          cmapwidget.cpp
                      -------------------
    description          : Map Display Widget
    begin                : Fri Oct 22 1999                                           
    copyright            : (C) 1999 by John-Paul Stanford                      
    email                : john-paul.stanford@virgin.net                       
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include "cmapwidget.h"
#include <stdio.h>

#define ROOM_SIZE      20
#define HALF_ROOM_SIZE 10

CMapRoom::CMapRoom(int type ,QWidget *parent=0, const char *name=0 ) : QWidget( parent, name )
{
	roomType=type;
	nw = -1; n = -1; ne = -1; e = -1;
	se = -1; s = -1; sw = -1; w = -1;	
	switch (roomType)
	{
		case 0 : setBackgroundMode( PaletteBackground );
						 setPalette(lightGray);
						 break;
		case 1 : setBackgroundMode( PaletteBackground );
						 setPalette(QColor(214,187,255));																
						 break;
	}
}

void CMapRoom::paintEvent( QPaintEvent *)
{
	QPainter p(this);
	p.setPen(black);
	int maxx = width()-1;
	int maxy = height()-1;
	switch (roomType)
	{
		case 0 : p.setPen(white);
						 p.drawLine(0,maxy,0,0);
						 p.drawLine(0,0,maxx,0);
						 p.setPen(black);
						 p.drawLine(1,maxy,maxx,maxy);
						 p.drawLine(maxx,maxy,maxx,1);
						 break;
 	  case 1 : p.setPen(black);
						 p.drawRect(0,0,maxx+1,maxy+1);
				     break;
	}	
}

CMapWidget::CMapWidget(QWidget *parent ) : QScrollView(parent,"mapwidget")
{
	setVScrollBarMode(QScrollView::Auto);
	setHScrollBarMode(QScrollView::Auto);
	
	// Setup default values
	XGridSpace = 20;
	YGridSpace = 20;
	hasGrid = true;
	gridColour = lightGray;
	resizeContents(4000,3000);
	setMapBackgroundColour(white);	
}

CMapWidget::~CMapWidget()
{
}

/** draw the contentets of the viewable part of the map widget */
void CMapWidget::drawContentsOffset(QPainter* paint, int ox, int oy, int cx, int cy, int cw, int ch)
{
	int x = 0;
	int y = 0;
	int y1,x1,x2,y2;
	int maxx= contentsWidth(),maxy = contentsHeight();
	QPainter* p= paint;	
	CMapPath *path;

  p->setClipRect(cx-ox, cy-oy, cw, ch);

	// Draw the grid if it's visable
	if (hasGrid)
	{
		p->setPen(gridColour);
	 	// Draw the lines going across
		for (y=0;y<=maxy;y+=XGridSpace)
		{
			x1 = 0;
			y1 = y;
			x2 = maxx;
			y2 = y;
			if (x1 < cx) x1=cx;
			if (y1 < cy) y1=cy;
			if (x2 > cx+cw-1) x2=cx+cw-1;
			if (y2 > cy+ch-1) y2=cy+ch-1;

			// Translate to scrolled coordinates...
			x1 -= ox;
			x2 -= ox;
			y1 -= oy;
			y2 -= oy;
	
			if ( x2 >= x1 && y2 >= y1 )
				p->drawLine(x1,y1,x2,y2);
		}	
	
		// Draw the lines going down
		for (x=0;x<=maxx;x+=YGridSpace)
		{
   		x1 = x;
			y1 = 0;
			x2 = x;
			y2 = maxy;
			if (x1 < cx) x1=cx;
			if (y1 < cy) y1=cy;
			if (x2 > cx+cw-1) x2=cx+cw-1;
			if (y2 > cy+ch-1) y2=cy+ch-1;

			// Translate to scrolled coordinates...
			x1 -= ox;
			x2 -= ox;
			y1 -= oy;
			y2 -= oy;

      if ( x2 >= x1 && y2 >= y1 )
				p->drawLine(x1,y1,x2,y2);
		}	
	}	

	// Draw paths
	p->setPen(black);
	for ( path=pathList.first(); path != 0; path=pathList.next() )
	{ 	
		x1 = (((path->getSrcX() * ROOM_SIZE) - ROOM_SIZE) * 2) + HALF_ROOM_SIZE;
		y1 = (((path->getSrcY() * ROOM_SIZE) - ROOM_SIZE) * 2) + HALF_ROOM_SIZE;
		x2 = (((path->getDestX() * ROOM_SIZE) - ROOM_SIZE) * 2) + HALF_ROOM_SIZE;
		y2 = (((path->getDestY() * ROOM_SIZE) - ROOM_SIZE) * 2) + HALF_ROOM_SIZE;
			if (x1 < cx) x1=cx;
			if (y1 < cy) y1=cy;
			if (x2 > cx+cw-1) x2=cx+cw-1;
			if (y2 > cy+ch-1) y2=cy+ch-1;

			// Translate to scrolled coordinates...
			x1 -= ox;
			x2 -= ox;
			y1 -= oy;
			y2 -= oy;

      if ( x2 >= x1 && y2 >= y1 )
				p->drawLine(x1,y1,x2,y2);
	}

}

/** Set the grid spaceing on the X Axis */
void CMapWidget::setGridXSpacing(int space)
{
	XGridSpace = space;
}

/** Set the grid spaceing on the X Axis */
void CMapWidget::setGridYSpacing(int space)
{
	YGridSpace = space;
}

/** Turns the grid on or off */
void CMapWidget::setGridVisable(bool visable)
{
	hasGrid = visable;
}

/** Returns the grid spaceing on the X Axis */
int CMapWidget::GridXSpaceing(void)
{
	return XGridSpace;
}

/** Returns the grid spaceing on the Y Axis */
int CMapWidget::GridYSpaceing(void)
{
	return YGridSpace;
}

/** Returns the visable state of the grid */
bool CMapWidget::GridVisable(void)
{
	return hasGrid;
}

/** Sets the colour of the grid */
void CMapWidget::setGridColour(QColor colour)
{	
	gridColour = colour;
}

/** Returnt the color of the grid */
QColor CMapWidget::GridColour(void)
{
	return gridColour;
}


/** This method sets the background colour of the map */
void CMapWidget::setMapBackgroundColour(QColor colour)
{
	viewport()->setBackgroundMode( PaletteBackground );
	viewport()->setPalette( QPalette( colour ) );
	backColour = colour;
}

/** This method returns the current backgroud colour of the map */
QColor CMapWidget::getMapBackgroundColour(void)
{
	return backColour;
}

/** This method is used to create a new room on the map */
void CMapWidget::createRoom(int x,int y,int type)
{
	CMapRoom *room;
	
	// Create the new room
	room = new CMapRoom(type,this,"room");
	room->resize(ROOM_SIZE,ROOM_SIZE);
	addChild(room,((x * ROOM_SIZE) - ROOM_SIZE) * 2,((y * ROOM_SIZE) - ROOM_SIZE) *2);

	// Add the room to the list
	roomList.append( room );
}

/** This method is used to create a path between to rooms */
void CMapWidget::createPath(int srcX,int srcY,int destX,int destY,int srcToDest,int destToSrc)
{
	// Create the new path
	pathList.append( new CMapPath(srcX,srcY,destX,destY,srcToDest,destToSrc));
}

/** This method is used to load a map */
void CMapWidget::loadMap(void)
{
	// Setup Example Map
	// Create Rooms
	createRoom(5,1,0);
	createRoom(5,2,0);
	createRoom(5,3,0);
	createRoom(5,4,0);
	createRoom(5,5,0);
	createRoom(5,6,0);
	createRoom(5,7,0);
	createRoom(5,8,0);
	createRoom(5,9,0);
	createRoom(5,10,0);
	createRoom(5,11,0);
	createRoom(5,12,0);
	createRoom(5,13,0);
	createRoom(5,14,0);
	createRoom(5,15,0);
	createRoom(6,15,0);
	createRoom(4,7,0);
	createRoom(4,8,0);
	createRoom(4,9,0);
	createRoom(4,10,0);
	createRoom(4,11,0);
	createRoom(3,11,0);
	createRoom(2,10,0);
	createRoom(2,9,0);
	createRoom(3,9,0);
	createRoom(3,8,0);
	createRoom(3,7,0);
	createRoom(2,7,0);
	createRoom(1,7,0);
	createRoom(2,7,0);	
	createRoom(2,6,0);
	createRoom(2,5,0);
	createRoom(3,5,0);

	// Create Paths
	createPath(5,1,5,2,1,1);
	createPath(5,2,5,3,1,1);
	createPath(5,3,5,4,1,1);
	createPath(5,4,5,5,1,1);
	createPath(5,5,5,6,1,1);
	createPath(5,6,5,7,1,1);
	createPath(5,7,5,8,1,1);
	createPath(5,8,5,9,1,1);
	createPath(5,9,5,10,1,1);
	createPath(5,10,5,11,1,1);
	createPath(5,11,5,12,1,1);
	createPath(5,12,5,13,1,1);
	createPath(5,13,5,14,1,1);
	createPath(5,14,5,15,1,1);
	createPath(5,15,6,15,1,1);
	createPath(4,7,5,7,1,1);
	createPath(3,7,4,7,1,1);
	createPath(2,7,3,7,1,1);
	createPath(1,7,2,7,1,1);
 	createPath(4,11,5,11,1,1);
	createPath(3,11,4,11,1,1);
	createPath(4,10,5,10,1,1);
	createPath(2,10,4,10,1,1);
	createPath(2,9,2,10,1,1);
	createPath(2,9,3,9,1,1);
	createPath(2,5,3,5,1,1);
	createPath(2,5,2,6,1,1);
	createPath(2,6,2,7,1,1);
	createPath(3,8,4,8,1,1);
	createPath(4,8,5,8,1,1);
	createPath(3,8,4,8,1,1);
	createPath(3,8,4,8,1,1);
	createPath(4,9,5,9,1,1);
	
}








