/***************************************************************************
                          mapwidget.h
                      -------------------
    description          : Map Display Widget
    begin                : Wed Oct 20 1999                                           
    copyright            : (C) 1999 by John-Paul Stanford                      
    email                : john-paul.stanford@virgin.net                       
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#ifndef CMAPWIDGET_H
#define CMAPWIDGET_H

#include <qscrollview.h>
#include <qpixmap.h>
#include <qlist.h>

/**This class is used to store the details of each path on
  *the map (The exits from one room to another)
  *@author John-Paul Stanford
  */

class CMapPath
{
public:
	/** This is the constuctor for the map paths and
    * used to create a path
    */
	CMapPath (int SrcX,int SrcY,int DestX,int DestY,int SrcToDest, int DestToSrc)
	{
		srcX = SrcX;
		srcY = SrcY;
		destX = DestX;
	  destY = DestY;
		srcToDest = SrcToDest;
		destToSrc = DestToSrc;
	}

	/** Get the start and finish locations of the path */
	int getDestX(void)     { return destX; }
	int getDestY(void)     { return destY; }
	int getSrcX(void)      { return srcX; }
	int getSrcY(void)      { return srcY; }	

	/** Get the directions a path can be traveled */
	int getSrcToDest(void) { return srcToDest; }
	int getDestToSrc(void) { return destToSrc; }

private:
	/** Thses are used to store the start of the path */
	int srcX,srcY;
	/** These are used to store the end of the path */
	int destX,destY;
	/** These are used to store the directions a path can be traveled in */
	bool srcToDest,destToSrc;
};

/**This class is used to store the details of each room
  *on the map
  *@author John-Paul Stanford
  */
class CMapRoom  : public QWidget
{
public:
	/** Constructor which is used to create a new room */
	CMapRoom( int type ,QWidget *parent=0, const char *name=0 );

	/** Uesd tp set the room type */
	void setType(int type)								{ roomType = type; }
	/** Used to get the room type */	
	int  getType(void)										{ return roomType; }
protected:
   void paintEvent( QPaintEvent * );
private:
	int nw,n,ne,e,se,s,sw,w;

 	/** Used to store the type of room */
	int roomType;
};

/**This is the widget used to display the map
  *@author John-Paul Stanford
  */
class CMapWidget : public QScrollView
{
	Q_OBJECT
public:
	/** The map widget constructor */
	CMapWidget(QWidget *parent=0);
	/** The Map widget deconstructor */
	~CMapWidget();
  /** Returns the grid spaceing on the X Axis */
  int GridXSpaceing(void);
  /** Returns the grid spaceing on the Y Axis */
  int GridYSpaceing(void);
  /** Returns the visable state of the grid */
  bool GridVisable(void);
  /** Turns the grid on or off */
  void setGridVisable(bool visable);
  /** Set the grid spaceing on the X Axis */
  void setGridYSpacing(int space);
  /** Set the grid spaceing on the X Axis */
  void setGridXSpacing(int space);
  /** Returnt the color of the grid */
  QColor GridColour(void);
  /** Sets the colour of the grid */
  void setGridColour(QColor colour);
  /** This method sets the background colour of the map */
  void setMapBackgroundColour(QColor colour);
	/** This method returns the current backgroud colour of the map */
	QColor getMapBackgroundColour(void);
	/** This method is used to load a map */
	void loadMap(void);

protected:
	/** draw the contentets of the viewable part of the map widget */
 	void drawContentsOffset(QPainter* paint, int ox, int oy, int cx, int cy, int cw, int ch);

private:	
	/** This method is used to create a new room on the map */
	void createRoom(int x,int y,int type);
	/** This method is used to create a path between to rooms */
	void createPath(int srcX,int srcY,int destX,int destY,int srcToDest,int destToSrc);
	/** This holds the current background color */
	QColor backColour;
	/** This is a list of all the rooms on the map */
	QList<CMapRoom> roomList;
	/** This is a list of all the paths on the map */
	QList<CMapPath> pathList;
  /** The spaceing of the gird on the Y axis */
  int YGridSpace;
  /** The spaceing of the gird on the X axis */
  int XGridSpace;
  /** Used to store the on/off state of the grid */
  int hasGrid;
	/** The Grid colour */
	QColor gridColour;
};

#endif






















