/***************************************************************************
                          cmapwindow.cpp
                      -------------------
    description          : Auto Mapper Window Class
    begin                : Fri Oct 22 1999                                           
    copyright            : (C) 1999 by John-Paul Stanford                      
    email                : john-paul.stanford@virgin.net                       
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include "cmapwindow.h"

#include <kmsgbox.h>


CMapWindow::CMapWindow()
{
	setCaption("Kmud Mapper");


  ///////////////////////////////////////////////////////////////////
  // read the config file options
  // read them _before_ the inits so we init the right
  readOptions();

  this->resize(mapperWindowWidth,mapperWindowHeight);

	initMenubar();
	initStatusBar();

  if (!bViewStatusbar)
    enableStatusBar(KStatusBar::Hide);

  menuBar()->setMenuBarPos(menu_bar_pos);

	map = new CMapWidget(this);
	map->setGridXSpacing(20);
	map->setGridYSpacing(20);
	map->setGridVisable(true);
	map->setGridColour(lightGray);
	map->setMapBackgroundColour(QColor(214,187,255));
	// map->setMapBackgroundColour(QColor(250,250,250));
	map->loadMap();
	setView(map,FALSE);
	show();
}

CMapWindow::~CMapWindow()
{
	delete file_menu;
}

void CMapWindow::readOptions(void)
{
  ///////////////////////////////////////////////////////////////////
  // read the config file entries
  KConfig *config = kapp->getConfig();

  config->setGroup("MAPPER APPEARANCE");
  mapperWindowWidth = config->readNumEntry("WindowWidth",600);
  mapperWindowHeight = config->readNumEntry("WindowHeight",400);
  bViewStatusbar = config->readBoolEntry("ShowStatusbar", true);

  menu_bar_pos = (KMenuBar::menuPosition)config->readNumEntry("MenuBarPos", KMenuBar::Top);
	
}

void CMapWindow::saveOptions(void)
{
	///////////////////////////////////////////////////////////////////
	// save the options to the config file.
  KConfig *config = kapp->getConfig();

  config->setGroup("MAPPER APPEARANCE");
  config->writeEntry("WindowWidth",this->width());
  config->writeEntry("WindowHeight",this->height());
	config->writeEntry("MenuBarPos", (int)menuBar()->menuBarPos());
  config->writeEntry("ShowStatusbar",statusBar()->isVisible());
}

void CMapWindow::initMenubar(void)
{
	///////////////////////////////////////////////////////////////////
  // menuBar entry file_menu
  file_menu = new QPopupMenu();
	  file_menu->insertItem(Icon("mini/save.xpm"), i18n("&Export Map"), ID_MAPPER_FILE_EXPORT );
  file_menu->insertSeparator();
  file_menu->insertItem(i18n("C&lose Window"), ID_MAPPER_FILE_CLOSE);

  // options_menu key accelerators
  file_menu->setAccel(CTRL+Key_C, ID_MAPPER_FILE_CLOSE);

	///////////////////////////////////////////////////////////////////
  // MENUBAR CONFIGURATION
  // set menuBar() the current menuBar and the position due to config file
  menuBar()->insertItem(i18n("&File"), file_menu);

	CONNECT_CMD(file_menu);
}

void CMapWindow::initStatusBar(void)
{
	statusBar()->setInsertOrder(KStatusBar::RightToLeft);
	statusBar()->insertItem(i18n("Ready."), ID_MAPPER_STATUS_MSG );
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION

void CMapWindow::commandCallback(int id_){
  switch (id_){
    ON_CMD(ID_MAPPER_FILE_EXPORT,          slotFileExport())
    ON_CMD(ID_MAPPER_FILE_CLOSE,           slotFileCloseWindow())
  }
}

void CMapWindow::statusCallback(int id_){
  switch (id_){
    ON_STATUS_MSG(ID_MAPPER_FILE_EXPORT,   i18n("Export the map to a different format"))

    ON_STATUS_MSG(ID_MAPPER_FILE_CLOSE, i18n("Close the map window"))
  }
}

void CMapWindow::slotStatusMsg(const char *text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message permanently
  statusBar()->clear();
  statusBar()->changeItem(text, ID_STATUS_MSG );
}


void CMapWindow::slotStatusHelpMsg(const char *text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message of whole statusbar temporary (text, msec)
  statusBar()->message(text, 2000);
}


void CMapWindow::slotFileCloseWindow()
{
  close();
}

void CMapWindow::slotFileExport()
{
	printf("Not yet implemented!\n");
}












