/***************************************************************************
                          cmudlineedit.h
                       -------------------
    description          : Class implementing an input line
    begin                : Wed Sep 8 1999
    copyright            : (C) 1999 by Stephan Uhlmann                         
    email                : suhlmann@gmx.de                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#ifndef CMUDLINEEDIT_H
#define CMUDLINEEDIT_H


#include <qlineedit.h>
#include <qkeycode.h>
#include <qstrlist.h>
#include "kmuddoc.h"

/**
	* Class implementing an input line. Supports history and the feature to keep the last input
	* selected rather than deleting it.
  *@author Stephan Uhlmann
  */

class CMudLineEdit : public QLineEdit  {
	Q_OBJECT
public:
	/** constructor */
	CMudLineEdit(KmudDoc* doc, QWidget * parent=0, const char * name=0);
	/** destructor */
	~CMudLineEdit();

	/** sets the maximum number of lines kept in the history  */
	void setMaxHistoryLines(int);
	/** returns true if the option for keeping the last input is set */
	bool getKeepLastInput();
	/** set/unsets the behaviour to keep the last input selected rather than deleting it */
	void setKeepLastInput(bool);
	/** gets the maximum number of lines kept in history */
	int getMaxHistoryLines();
	/** sets the current position in the history */
	void setHistoryPosition(int);
	/** gets the current position in the history. */
	int getHistoryPosition();
	/** sets the to complete input for use with history completion */
	int setCompletionInput(QString);
	/** gets the to complete input */
	QString getCompletionInput();
  /** Scrolls in the history according 
to the pressed key. */
  void historyScroll(QKeyEvent * e);

protected:
	/** redefined keyPressevent to get special keys */
	void keyPressEvent(QKeyEvent*);
	/** moves the history in the input line one item up */
	void historyUp();
	/** moves the history in the input line one item down */
	void historyDown();

	void CMudLineEdit::addInputToHistory(QString);

private:
	QStrList history;
	QString completionInput;
	int historyPosition;
	int maxHistoryLines;
	bool keepLastInput;

  KmudDoc* document;
    	
signals:
	/** emitted when pageup is pressed */
	void scrollKeyPressed(QKeyEvent*);
  /** emitted when a macro is pressed */
  void macroPressed(QString);
};

#endif














