/***************************************************************************
                          cmudview.cpp
                      -------------------
		description					 : class providing a colored output view
    begin                : Wed Sep 8 1999
    copyright            : (C) 1999 by Stephan Uhlmann
    email                : suhlmann@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include "cmudview.h"

#include <qtableview.h>
#include <qpixmap.h>
#include <qregexp.h>

// for copy&paste
#include <qclipboard.h>
#include <qapplication.h>

#include <kapp.h>


CMudView::CMudView(KmudDoc* document, QWidget * parent, const char * name)
 : QTableView(parent,name)
{
    doc = document;

 init();

 setMaxLines(5000);

 setForeground(QColor(0,0,0));
 setBackground(QColor(255,255,255));
 setFont(QFont("fixed"));

	// for cut&paste
	markedAreaStartRow = -1;
	markedAreaStartCol = -1;
	markedAreaEndRow = -1;
	markedAreaEndCol = -1;
	markedText = false;

}


CMudView::~CMudView()
{
}


void CMudView::init()
{
 MudViewLineData* ld;

 setAutoUpdate(false);

 setTopLeftCell(0,0);
 lineX=0;

 data.setAutoDelete(true);
 data.clear();


 ld = new MudViewLineData;
 ld->chunks.setAutoDelete(true);
 ld->chunks.clear();
 ld->lineHeight=0;
 data.append(ld);

 setNumRows(1);
 setNumCols(1);
 updateTableSize();


 setBackgroundColor(defaultBackground);
 setTableFlags( Tbl_autoVScrollBar );
 setFrameStyle( QFrame::WinPanel | QFrame::Sunken );
 setCellHeight(ld->lineHeight);
 setCellWidth(viewWidth());


 setAutoUpdate(true);
 repaint();

}


void CMudView::paintCell(QPainter* painter, int row, int col )
{
 QRect rect = cellUpdateRect();
 QPixmap* buf;
 QPainter* p;
 int x=0;
 int c = 0;
 uint l;
 int col1 = 0;
 int col2 = 0;
 MudViewChunkData* chunk;
 QFontMetrics* fm;


 buf = new QPixmap;

 buf->resize(cellWidth(col),cellHeight(row));
 buf->fill(defaultBackground);

 p = new QPainter (buf);
 p->setBackgroundMode(OpaqueMode);

 chunk=data.at(row)->chunks.first();

 while (chunk!=NULL)
  {
	 if (rowMarked (row) >= 0)
	 {
   	p->setPen(chunk->bgc);
   	p->setBackgroundColor(chunk->fgc);
	 } // if
	 else
	 {
   	p->setPen(chunk->fgc);
   	p->setBackgroundColor(chunk->bgc);
	 } // else
   p->setFont(chunk->font);
   fm = new QFontMetrics(chunk->font);
	 if (rowMarked (row) == 0)
	 {
		if (row == markedAreaStartRow)
		{
			col1 = markedAreaStartCol;
			col2 = 1000;
		} // if
		if (row == markedAreaEndRow)
		{
			col2 = markedAreaEndCol;
			if (row != markedAreaStartRow) col1 = 0;
		} // if
		if (markedAreaStartRow > markedAreaEndRow)
		{
			if (row == markedAreaStartRow) col2 = 0;
			if (row == markedAreaEndRow) col1 = 1000;
		} // if
		if (col1 > col2)
		{
			int save = col1;
			col1 = col2;
			col2 = save;
		} // if
		for (l = 0;  l < chunk->s.length (); l++)
		{
			if (((c + (int)l) < col1) || ((c + (int)l) >= col2))
			{
   			p->setPen(chunk->fgc);
   			p->setBackgroundColor(chunk->bgc);
			} // if
			else
			{
   			p->setPen(chunk->bgc);
   			p->setBackgroundColor(chunk->fgc);
			} // else
			p->drawText(x,0,buf->width()-x,buf->height(), AlignLeft | AlignTop, chunk->s.mid (l, 1));
			x=x+fm->width(chunk->s.mid(l, 1));
		} // for
	 } // if
	 else
	 {
   	p->drawText(x,0,buf->width()-x,buf->height(),
                AlignLeft | AlignTop, chunk->s);

    x=x+fm->width(chunk->s);
	 } // else
	 c = c + chunk->s.length ();
   delete fm;

   chunk=data.at(row)->chunks.next();
  }
 delete p;


 painter->drawPixmap(rect.topLeft(), *buf, rect);

 delete buf;

}



void CMudView::resizeEvent(QResizeEvent* e)
{
 QTableView::resizeEvent(e);
 setCellWidth(viewWidth());
}



void CMudView::append(QString s, QColor foreground, QColor background, QFont font)
{
 QString t,u,real;
 QString tabSpace;
 int tabSpaceI;
 int i=0;
 char c;
 MudViewChunkData* cd;
 MudViewLineData* ld;
 QFontMetrics* fm;
 int ansiStart=0;
 int ansiEnd=0;
 QString ansiCode, ansiCodeChunk;
 QString ansiCodeAttrib;
 int ansiI;
 setAutoUpdate(false); // avoid flickering
 QString ansiAttrib[8];
 int lastAttrib = 0, count = 0;
 bool useBold = false;
 bool useBright = false;
 bool colorUsed = false;
 bool useNormal = true;
 bool useUnderline = false;
 bool useBlack = false, useRed = false, useGreen = false, useYellow = false;
 bool useBlue = false, useMagenta = false, useCyan = false, useWhite = false;


t="";
 for (i=0;i<(int)s.size();i++)
  {
   c=s.at(i);
   switch (c)
   {
    case '\007' : if (useBeep == true)
                      KApplication::beep();
                  break;

    case '\r'   : break; // skip all \r dos-cr's

    case '\t'   : // expand tabs to spaces
                  tabSpace="";
                  for (tabSpaceI=0;tabSpaceI<(8-lineX%8);tabSpaceI++)
                  { tabSpace+=" "; }
                  s.replace(i,1,tabSpace);
                  i--;lineX--;
                  break;

    case '\n'   : // flush the chunk and make an new line
                  cd = new MudViewChunkData;
                  if (foreground==defaultForeground) cd->fgc=actualForeground; else cd->fgc=foreground;
                  if (background==defaultBackground) cd->bgc=actualBackground; else cd->bgc=background;
                  if (font==defaultFont) {cd->font=actualFont;} else {cd->font=font; actualFont=font;}
                  cd->s=t;
                  (data.last())->chunks.append(cd);
                  fm = new QFontMetrics(font);
                  if (fm->height() > data.last()->lineHeight) data.last()->lineHeight=fm->height();
                  delete fm;
                  t="";
                  // new line
                  ld = new MudViewLineData;
                  (ld->chunks).clear();
                  ld->lineHeight=0;
                  data.append(ld);
                  setNumRows(numRows()+1);
                  updateTableSize();

                  lineX=0;
                  break;

    case '\x1B' :   // flush chunk and do the ansi/vt100 escape code interpretation
            cd = new MudViewChunkData;
            if (foreground==defaultForeground) cd->fgc=actualForeground; else cd->fgc=foreground;
            if (background==defaultBackground) cd->bgc=actualBackground; else cd->bgc=background;
            if (font==defaultFont) {cd->font=actualFont;} else {cd->font=font; actualFont=font;}
            cd->s=t;
            (data.last())->chunks.append(cd);
            fm = new QFontMetrics(font);
            if (fm->height() > data.last()->lineHeight) data.last()->lineHeight=fm->height();
            delete fm;
            t="";

            ansiStart=i;

            /** Find the end of the ANSI chunk */
            ansiEnd=s.find('m',ansiStart,true);

            /** WARNING: it might be that there is an escape in the telnet buffer but the
             ** appropiate 'm' is in the next buffer.  Fixed that in CTelnet, but one "if"
             ** more does no harm...
             **/
            if (ansiEnd < ansiStart)
                break; /** sanity check */

            /** The code chunk */
            ansiCode = s.mid(ansiStart+2,ansiEnd-ansiStart-1);
            ansiCodeChunk = ansiCode;

            /** The end of the code chunk */
            i = ansiEnd;

            /** This loop gets the ansi codes into an array */
            count = 0;
            while ((ansiCode.isEmpty()==false) && (count < 20))
            {
            	  ansiI = ansiCode.find(QRegExp("[;m]"));
            	  ansiAttrib[count] = ansiCode.mid(0, ansiI);
            	  count++;
             	  ansiCode = ansiCode.mid(ansiI + 1, ansiCode.length() - ansiI - 1);
            }

            lastAttrib = count;
            count = 0;

            switch (interpretType) {
                case 0: /** DEFAULT -- detects ANSI/VT100 for you */
                case 1: /** ANSI -- Force ANSI -- Combined 0 and 1 because they're so similar */

                        /** \x1B[2J Erases the screen with the background color and moves the cursor home.
                         ** According to ANSI Standard, that sequence "Clears screen and homes cursor".
                         **/
                        if ((s.at(ansiStart + 1) == '[') && (s.at(ansiStart + 2) == '2') && (s.at(ansiStart + 3) == 'J'))
                        {
                            erase();
                            init();
                            i = ansiStart + 6;
                            break;
                        }

                        /** Loop through and find the values 0-9 */
                        while (count < lastAttrib)
                        {
                            if ((ansiAttrib[count].toInt() >= 0) && (ansiAttrib[count].toInt() <= 9))
                            {
                                if (ansiAttrib[count].toInt() == 0) {
                                    actualBackground = defaultBackground;
                                    useBold = false;
                                    useBright = false;
                                    useUnderline = false;
                                    colorUsed = false;
                                    useNormal = true;

                                    useBlack = false;
                                    useRed = false;
                                    useGreen = false;
                                    useYellow = false;
                                    useBlue = false;
                                    useMagenta = false;
                                    useCyan = false;
                                    useWhite = false;
                                } else if (ansiAttrib[count].toInt() == 1) {
                                    if (colorUsed == true) {
                                        useBright = true;
                                        useBold = false;
                                        colorUsed = true;
                                        useNormal = false;
                                    }
                                    else {
                                        if (interpretType == 1){
                                            useBright = true;
                                            useBold = false;
                                            colorUsed = false;
                                            useNormal = false;
                                        } else {
                                            useBright = false;
                                            useBold = true;
                                            colorUsed = false;
                                            useNormal = false;
                                        }
                                    }
                                } else if (ansiAttrib[count].toInt() == 2) {
                                    useBold = false;
                                    useBright = false;
                                    useNormal = false;
                                } else if (ansiAttrib[count].toInt() == 4) {
                                    useUnderline = true;
                                    useNormal = false;
                                } else if (ansiAttrib[count].toInt() == 5) {
                                    /** Insert blinking code here */
                                } else if (ansiAttrib[count].toInt() == 7) {
                                    actualForeground = defaultBackground;
                                    actualBackground = defaultForeground;
                                    useNormal = false;
                                } else if (ansiAttrib[count].toInt() == 8) {
                                    actualForeground = defaultBackground;
                                    useNormal = false;
                                }
                            }
                            count++;
                        } /** end while */

                        count = 0;

                        while (count < lastAttrib)
                        {
                            if ((ansiAttrib[count].toInt() >= 30) && (ansiAttrib[count].toInt() <= 37))
                            {
                                actualForeground = defaultForeground;
                                colorUsed = true;
                                useNormal = false;
                                actualFont.setBold(false);

                                if (useBold == true) {
                                    useBright = true;
                                    useBold = false;
                                }
                                else {
                                    useBold = false;
  //                                  useBright = false;
                                }
                            }
                            count++;
                        }

                        count = 0;
                        /** Background colors */
                        while (count < lastAttrib)
                        {
                            if ((ansiAttrib[count].toInt() >= 40) && (ansiAttrib[count].toInt() <= 47))
                            {
                                actualBackground = defaultBackground;
                                switch (ansiAttrib[count].toInt()) {
                                    case 40: actualBackground = BG_BLACK;
                                             break;
                                    case 41: actualBackground = BG_RED;
                                             break;
                                    case 42: actualBackground = BG_GREEN;
                                             break;
                                    case 43: actualBackground = BG_YELLOW;
                                             break;
                                    case 44: actualBackground = BG_BLUE;
                                             break;
                                    case 45: actualBackground = BG_MAGENTA;
                                             break;
                                    case 46: actualBackground = BG_CYAN;
                                             break;
                                    case 47: actualBackground = BG_WHITE;
                                             break;
                                }
                            }
                            count++;
                        }

                        count = 0;

                        while (count < lastAttrib)
                        {
                            if ((ansiAttrib[count].toInt() >= 30) && (ansiAttrib[count].toInt() <= 37))
                            {
                                useBlack = false; useRed = false; useGreen = false; useYellow = false;
                                useBlue = false; useMagenta = false; useCyan = false; useWhite = false;

                                switch (ansiAttrib[count].toInt()) {
                                    case 30: useBlack =true;    break;
                                    case 31: useRed = true;     break;
                                    case 32: useGreen = true;   break;
                                    case 33: useYellow = true;  break;
                                    case 34: useBlue = true;    break;
                                    case 35: useMagenta = true; break;
                                    case 36: useCyan = true;    break;
                                    case 37: useWhite = true;   break;
                                }
                            }
                            count++;
                        }

                        if (useBold == true)
                            actualFont.setBold(true);
                        if (useBold == false)
                            actualFont.setBold(false);
                        if (useBright == true)
                            actualFont.setBold(false);
                        if (useNormal == true) {
                            actualForeground=defaultForeground;
                            actualBackground=defaultBackground;
                            actualFont=defaultFont;
                        }
                        if (useUnderline == true)
                            actualFont.setUnderline(true);

                        /** Black / Gray */
                        if ((useBlack == true) && (useBright == false)) {
                            actualForeground = BLACK;
                        }
                        if ((useBlack == true) && (useBright == true)) {
                            actualForeground = GRAY;
                        }
                        /** Red */
                        if ((useRed == true) && (useBright == false)) {
                            actualForeground = DARK_RED;
                        }
                        if ((useRed == true) && (useBright == true)) {
                            actualForeground = LIGHT_RED;
                        }
                        /** Green */
                        if ((useGreen == true) && (useBright == false)) {
                            actualForeground = DARK_GREEN;
                        }
                        if ((useGreen == true) && (useBright == true)) {
                            actualForeground = LIGHT_GREEN;
                        }
                        /** Yellow */
                        if ((useYellow == true) && (useBright == false)) {
                            actualForeground = DARK_YELLOW;
                        }
                        if ((useYellow == true) && (useBright == true)) {
                            actualForeground = LIGHT_YELLOW;
                        }
                        /** Blue */
                        if ((useBlue == true) && (useBright == false)) {
                            actualForeground = DARK_BLUE;
                        }
                        if ((useBlue == true) && (useBright == true)) {
                            actualForeground = LIGHT_BLUE;
                        }
                        /** Magenta */
                        if ((useMagenta == true) && (useBright == false)) {
                            actualForeground = DARK_MAGENTA;
                        }
                        if ((useMagenta == true) && (useBright == true)) {
                            actualForeground = LIGHT_MAGENTA;
                        }
                        /** Cyan */
                        if ((useCyan == true) && (useBright == false)) {
                            actualForeground = DARK_CYAN;
                        }
                        if ((useCyan == true) && (useBright == true)) {
                            actualForeground = LIGHT_CYAN;
                        }
                        /** White */
                        if ((useWhite == true) && (useBright == false)) {
                            actualForeground = DARK_WHITE;
                        }
                        if ((useWhite == true) && (useBright == true)) {
                            actualForeground = LIGHT_WHITE;
                        }
                        break;
                case 2: /** VT100 -- forces VT100 interpretation */
                        count = 0;
                        while (count < lastAttrib)
                        {
                            if (ansiAttrib[count].toInt() == 0)
                                actualFont.setBold(false);
                            else if (ansiAttrib[count].toInt() == 1)
                                actualFont.setBold(true);
                            count++;
                        }
                        break;
                case 3: /** NONE -- eats ANSI codes */
                        break;
                case 4: /** DEBUG -- shows ANSI codes as BOLD*/
                        useDebugHighlight = true;
                        MudViewChunkData* v = new MudViewChunkData;

                        ansiCodeChunk = QString("[").append(ansiCodeChunk);

                        /** For choosing whether or not to highlight ANSI codes */
                        if (useDebugHighlight == true)
                            v->fgc = DEBUG_HIGHLIGHT;
                        else
                            v->fgc = defaultForeground;

                        if (background==defaultBackground) v->bgc=actualBackground; else v->bgc=background;
                        if (font==defaultFont) {v->font=actualFont;} else {v->font=font; actualFont=font;}
                        v->s=ansiCodeChunk;
                        (data.last())->chunks.append(v);
                        break;
            }
            break;


    case ' '    : t=t+c;
                  if (lineX>=90) {
                  cd = new MudViewChunkData;
                  if (foreground==defaultForeground) cd->fgc=actualForeground; else cd->fgc=foreground;
                  if (background==defaultBackground) cd->bgc=actualBackground; else cd->bgc=background;
                  if (font==defaultFont) {cd->font=actualFont;} else {cd->font=font; actualFont=font;}
                  cd->s=t;
                  (data.last())->chunks.append(cd);
                  fm = new QFontMetrics(font);
                  if (fm->height() > data.last()->lineHeight) data.last()->lineHeight=fm->height();
                  delete fm;
                  t="";
                  // new line
                  ld = new MudViewLineData;
                  (ld->chunks).clear();
                  ld->lineHeight=0;
                  data.append(ld);
                  setNumRows(numRows()+1);
                  updateTableSize();

                  lineX=0;
                  }
                  break;

    default     : t=t+c;
   }
   lineX++;
  }

if (!t.isEmpty())
 {

  cd = new MudViewChunkData;
  if (foreground==defaultForeground) cd->fgc=actualForeground; else cd->fgc=foreground;
  if (background==defaultBackground) cd->bgc=actualBackground; else cd->bgc=background;
  if (font==defaultFont) {cd->font=actualFont;} else {cd->font=font; actualFont=font;}
  cd->s=t;
  (data.last())->chunks.append(cd);
  fm = new QFontMetrics(font);
  if (fm->height() > data.last()->lineHeight) data.last()->lineHeight=fm->height();
  delete fm;
  t="";
 }


 // shrink view to maxLines
 setMaxLines(maxLines);



 setYOffset(maxYOffset());
 setAutoUpdate(true);
 repaint(false); // no need to erase, we do it in paintCell() anyway

}

int CMudView::lineLength(int line)
{
 return cellWidth(line);
}

int CMudView::numLines()
{
 return data.count()-1;
}


int CMudView::cellHeight(int row)
{
 if ((unsigned int)row > data.count()-1) return 0;
 else return (data.at(row)->lineHeight);
}




void CMudView::setForeground(QColor fgc)
{
 defaultForeground=fgc;
 actualForeground=fgc;
 repaint(false);
}

void CMudView::setBackground(QColor bgc)
{
 defaultBackground=bgc;
 actualBackground=bgc;
 setBackgroundColor(bgc);
 repaint(false);
}

void CMudView::setFont(QFont font)
{
 defaultFont=font;
 actualFont=font;
 repaint(false);
}

void CMudView::setInterpretType(int i)
{
  interpretType = i;
}


void CMudView::setDebugHighlightColor(QColor color)
{
    DEBUG_HIGHLIGHT = color;
}

// FOR ANSI COLORS
// Dark Colors
void CMudView::setOutputANSIBlack(QColor color)
{
 BLACK=color;
}
void CMudView::setOutputANSIDarkRed(QColor color)
{
 DARK_RED=color;
}
void CMudView::setOutputANSIDarkGreen(QColor color)
{
 DARK_GREEN=color;
}
void CMudView::setOutputANSIDarkYellow(QColor color)
{
 DARK_YELLOW=color;
}
void CMudView::setOutputANSIDarkBlue(QColor color)
{
 DARK_BLUE=color;
}
void CMudView::setOutputANSIDarkMagenta(QColor color)
{
 DARK_MAGENTA=color;
}
void CMudView::setOutputANSIDarkCyan(QColor color)
{
 DARK_CYAN=color;
}
void CMudView::setOutputANSIDarkWhite(QColor color)
{
 DARK_WHITE=color;
}

// Light Colors
void CMudView::setOutputANSIGray(QColor color)
{
 GRAY=color;
}
void CMudView::setOutputANSILightRed(QColor color)
{
 LIGHT_RED=color;
}
void CMudView::setOutputANSILightGreen(QColor color)
{
 LIGHT_GREEN=color;
}
void CMudView::setOutputANSILightYellow(QColor color)
{
 LIGHT_YELLOW=color;
}
void CMudView::setOutputANSILightBlue(QColor color)
{
 LIGHT_BLUE=color;
}
void CMudView::setOutputANSILightMagenta(QColor color)
{
 LIGHT_MAGENTA=color;
}
void CMudView::setOutputANSILightCyan(QColor color)
{
 LIGHT_CYAN=color;
}
void CMudView::setOutputANSILightWhite(QColor color)
{
 LIGHT_WHITE=color;
}

// background colors
void CMudView::setOutputBgBlack(QColor color)
{   BG_BLACK = color;    }
void CMudView::setOutputBgRed(QColor color)
{   BG_RED = color;    }
void CMudView::setOutputBgGreen(QColor color)
{   BG_GREEN = color;    }
void CMudView::setOutputBgYellow(QColor color)
{   BG_YELLOW = color;    }
void CMudView::setOutputBgBlue(QColor color)
{   BG_BLUE = color;    }
void CMudView::setOutputBgMagenta(QColor color)
{   BG_MAGENTA = color;    }
void CMudView::setOutputBgCyan(QColor color)
{   BG_CYAN = color;    }
void CMudView::setOutputBgWhite(QColor color)
{   BG_WHITE = color;    }

// END ANSI COLORS



QColor CMudView::getForeground()
{
 return defaultForeground;
}

QColor CMudView::getBackground()
{
 return defaultBackground;
}

QFont CMudView::getFont()
{
 return defaultFont;
}


int CMudView::getInterpretType()
{
  return interpretType;
}


// For Debug Highlighting
QColor CMudView::getDebugHighlightColor()
{
    return DEBUG_HIGHLIGHT;
}

// FOR ANSI COLORS
// Dark Colors
QColor CMudView::getOutputANSIBlack()
{
  return BLACK;
}
QColor CMudView::getOutputANSIDarkRed()
{
  return DARK_RED;
}
QColor CMudView::getOutputANSIDarkGreen()
{
  return DARK_GREEN;
}
QColor CMudView::getOutputANSIDarkYellow()
{
  return DARK_YELLOW;
}
QColor CMudView::getOutputANSIDarkBlue()
{
  return DARK_BLUE;
}
QColor CMudView::getOutputANSIDarkMagenta()
{
  return DARK_MAGENTA;
}
QColor CMudView::getOutputANSIDarkCyan()
{
  return DARK_CYAN;
}
QColor CMudView::getOutputANSIDarkWhite()
{
  return DARK_WHITE;
}

// Light Colors
QColor CMudView::getOutputANSIGray()
{
  return GRAY;
}
QColor CMudView::getOutputANSILightRed()
{
  return LIGHT_RED;
}
QColor CMudView::getOutputANSILightGreen()
{
  return LIGHT_GREEN;
}
QColor CMudView::getOutputANSILightYellow()
{
  return LIGHT_YELLOW;
}
QColor CMudView::getOutputANSILightBlue()
{
  return LIGHT_BLUE;
}
QColor CMudView::getOutputANSILightMagenta()
{
  return LIGHT_MAGENTA;
}
QColor CMudView::getOutputANSILightCyan()
{
  return LIGHT_CYAN;
}
QColor CMudView::getOutputANSILightWhite()
{
  return LIGHT_WHITE;
}
// background colors
QColor CMudView::getOutputBgBlack()
{    return BG_BLACK;   }
QColor CMudView::getOutputBgRed()
{    return BG_RED;   }
QColor CMudView::getOutputBgGreen()
{    return BG_GREEN;   }
QColor CMudView::getOutputBgYellow()
{    return BG_YELLOW;   }
QColor CMudView::getOutputBgBlue()
{    return BG_BLUE;   }
QColor CMudView::getOutputBgMagenta()
{    return BG_MAGENTA;   }
QColor CMudView::getOutputBgCyan()
{    return BG_CYAN;   }
QColor CMudView::getOutputBgWhite()
{    return BG_WHITE;   }


// END ANSI COLORS


void CMudView::setMaxLines(int i)
{
 if (i>0) maxLines=i; else maxLines=1;
 while (numLines()>maxLines)
  {
   data.removeFirst();
  }
  setNumRows(maxLines);
  updateTableSize();
}

void CMudView::slotScrollKey(QKeyEvent * k){
	setAutoUpdate(false);
	switch (k->key())
	{
		case Key_PageUp:
				setOffset(0,yOffset()-(viewHeight()/2));
				break;
		case Key_PageDown:
				if (yOffset()<maxYOffset())
				setOffset(0,yOffset()+(viewHeight()/2));
				break;
		case Key_Home:
				setOffset(0,0);
				break;
		case Key_End:
				setOffset(0,maxYOffset());
				break;
	}
	setAutoUpdate(true);
	repaint();
}
/** copies the marked text in the X-Clipboard */
void CMudView::copyMarkedText (){
	QClipboard* cb = QApplication::clipboard();
	if (markedText) cb->setText (getMarkedText ());
}
/** overwrites the mousePressEvent-Method from QWidget
implements the start of marking a text-area */
void CMudView::mousePressEvent (QMouseEvent* e){
	int oldStartRow = markedAreaStartRow;
	int oldEndRow = markedAreaEndRow;
	int l;
	markedText = false;
	if (oldStartRow > oldEndRow)
	{
		int save = oldStartRow;
		oldStartRow = oldEndRow;
		oldEndRow = save;
	} // if
	markedAreaStartRow = findRow (e->y ());
	if (markedAreaStartRow == -1)
	{
		markedAreaStartRow = numLines ()- 1;
		markedAreaStartCol = 0;
	} // if
	else
	{
		markedAreaStartCol = getCharacterNumber (markedAreaStartRow, e->x ());
	} // else
	markedAreaEndCol = markedAreaStartCol;
	markedAreaEndRow = markedAreaStartRow;
	for (l = oldStartRow; l <= oldEndRow; l++)
	{
		updateCell (l, 0);
	} // for
	updateCell (markedAreaStartRow, 0);

  QTableView::mousePressEvent(e);
}

/** gets the number of the character in the line row and on the position xPos */
int CMudView::getCharacterNumber (int Row, int xPos){
	MudViewChunkData* chunk = data.at(Row)->chunks.first();
	int x = 0;
	int n = 0;
	int Number = 0;
	uint l = 0;
	while (chunk!=NULL)
	{
		QFontMetrics* fm = new QFontMetrics (chunk->font);
		QString s = chunk->s;
		for (l = 0; l < s.length(); l++)
		{
			x = x + fm->width (s.mid (l, 1));
			if ((x > xPos) && (Number == 0))
			{
				Number = n;
			} // if
			n++;
		} // for
		delete fm;
		chunk=data.at(Row)->chunks.next();
	} // while
	return Number;
}

/** overwrites the mouseMoveEvent from QWidget
implements the marking of a area */
void CMudView::mouseMoveEvent (QMouseEvent* e){
	int oldRow = markedAreaEndRow;
	int newRow;
	int l;

	markedText = true;
	markedAreaEndRow = findRow (e->y ());
	if (markedAreaEndRow == -1)
	{
		markedAreaEndRow = numLines ();
		markedAreaEndCol = 10000;
	} // if
	else
	{
		markedAreaEndCol = getCharacterNumber (markedAreaEndRow, e->x ());
	} // else
	copyMarkedText ();
	newRow = markedAreaEndRow;
	if (oldRow > newRow)
	{
		int save = oldRow;
		oldRow = newRow;
		newRow = save;
	} // if
	for (l = oldRow; l <= newRow; l++)
	{
		updateCell (l, 0);
	} // for

  QTableView::mouseMoveEvent(e);
}

/** returns the text in the marked area */
QString CMudView::getMarkedText (){
	QString markedText = "";
	int row1 = markedAreaStartRow;
	int row2 = markedAreaEndRow;
	int col1 = markedAreaStartCol;
	int col2 = markedAreaEndCol;
	int l;
	MudViewChunkData* chunk;
  MudViewLineData* line;

	if (row1 > row2)
	{
		int save = row1;
		row1 = row2;
		row2 = save;
		save = col1;
		col1 = col2;
		col2 = save;
	} // if

  line=data.at(row1);
  if (!line) return markedText;
	chunk = line->chunks.first();
	l = 0;
	while (chunk != 0)
	{
		if (((l + (int) chunk->s.length ()) > col1) && ((chunk->s.length () - col1 + l) > 0))
		{
			markedText = markedText + chunk->s.right (chunk->s.length () - col1 + l);
		} // if
		l = l + chunk->s.length ();
		chunk = data.at(row1)->chunks.next();

	} // while
	markedText = markedText + "\n";
	for (l = row1 + 1; l < row2; l++)
	{
		chunk = data.at(l)->chunks.first();
		while (chunk != 0)
		{
			markedText = markedText + chunk->s;
			chunk = data.at(l)->chunks.next();
		} // while
		if (l != row2) markedText = markedText + "\n";
	} // for
	chunk = data.at(row2)->chunks.first();
	l = 0;
	while (chunk != 0)
	{
		if ((l < col2) && ((col2 - l) > 0))
		{
			markedText = markedText + chunk->s.left (col2 - l);
		} // if
		l = l + chunk->s.length ();
		chunk = data.at(row1)->chunks.next();
	} // while
	// row1 == row2
	if (row1 == row2)
	{
		markedText = "";
		if (col1 > col2)
		{
			int save1 = col1;
			col1 = col2;
			col2 = save1;
		} // if
		chunk = data.at(row1)->chunks.first();
		l = 0;
		while (chunk != 0)
		{
			if ((col2 - l - col1) > 0) markedText = markedText + chunk->s.mid (col1 - l, col2 - l - col1);
			l = l + chunk->s.length();
			chunk = data.at(row1)->chunks.next();
		} // while
	} // if
	return markedText;
}

/** returns true, if the specified row is marked */
int CMudView::rowMarked (int Row){
	int b = -1;
	if ((Row > markedAreaStartRow) && (Row < markedAreaEndRow)) b = 1;
	if ((Row < markedAreaStartRow) && (Row > markedAreaEndRow)) b = 1;
	if ((Row == markedAreaStartRow) || (Row == markedAreaEndRow)) b = 0;
	if (!markedText) b= -1;
	return b;
}

void CMudView::setUseBeep(bool u) {
    useBeep = u;
}

bool CMudView::getUseBeep() {
    return useBeep;
}