/***************************************************************************
                          dlgconnectionwizard.cpp  -  description
                             -------------------
    begin                : Sun Oct 17 1999
    copyright            : (C) 1999 by Stephan Uhlmann
    email                : suhlmann@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dlgconnectionwizard.h"

DlgConnectionWizard::DlgConnectionWizard( KmudDoc* document, QWidget *parent, const char *name) : QDialog(parent,name,true){


	doc=document;
  saveItem=NULL;

	initDialog();

	charDB_ListView->addColumn( i18n("Character") );
	charDB_ListView->addColumn( i18n("Mud") );
  charDB_ListView->setAllColumnsShowFocus( true );
  charDB_ListView->setColumnWidthMode( 0, QListView::Manual );
  charDB_ListView->setColumnWidthMode( 1, QListView::Manual );
  charDB_ListView->setColumnWidth( 0, charDB_ListView->width() / 2 );
  charDB_ListView->setColumnWidth( 1, charDB_ListView->width() / 2 );

	connect( connect_Button, SIGNAL(clicked()), SLOT(slotConnectPressed()) );
	connect( ok_Button, SIGNAL(clicked()), SLOT(slotOKPressed()) );
	connect( cancel_Button, SIGNAL(clicked()), SLOT(slotCancelPressed()) );
	connect( new_Button, SIGNAL(clicked()), SLOT(slotNewPressed()) );
	connect( delete_Button, SIGNAL(clicked()), SLOT(slotDeletePressed()) );

	connect( mudListSelect_Button, SIGNAL(clicked()), SLOT(slotSelectMudPressed()) );
	connect( charProfileSelect_Button, SIGNAL(clicked()), SLOT(slotSelectProfilePressed()) );

  connect( charDB_ListView, SIGNAL(selectionChanged(QListViewItem*)), SLOT(slotShowItem(QListViewItem*)) );

//  connect( login_LineEdit, SIGNAL(textChanged(const char*)), SLOT(slotChangedLogin(const char*)) );
//  connect( mudTitle_LineEdit, SIGNAL(textChanged(const char*)), SLOT(slotChangedMudTitle(const char*)) );


	updateListView();
  if (charDB_ListView->firstChild()!=NULL)
   charDB_ListView->setSelected(charDB_ListView->firstChild(),true);

  setDialogState();

}

DlgConnectionWizard::~DlgConnectionWizard(){
}

void DlgConnectionWizard::slotConnectPressed()
{
 applyInputTo(saveItem);
 doc->writeCharDB();
 done(1);
}

void DlgConnectionWizard::slotOKPressed()
{
 applyInputTo(saveItem);
 doc->writeCharDB();
 done(2);
}

void DlgConnectionWizard::slotCancelPressed()
{
 doc->readCharDB();
 reject();
}

void DlgConnectionWizard::slotNewPressed()
{
 CCharacterProfile* prof;
 CMudProfile* mud;

 prof=new CCharacterProfile;
 mud=new CMudProfile;

 doc->insertEntry("@","",prof,mud);

 updateListView();
 if (charDB_ListView->firstChild()!=NULL)
  charDB_ListView->setSelected(charDB_ListView->firstChild(),true);

 setDialogState();

 login_LineEdit->setFocus();
}

void DlgConnectionWizard::slotDeletePressed()
{
/* QListViewItem* item;

 item=charDB_ListView->firstChild();

 while (item!=NULL)
 {
  if (charDB_ListView->isSelected(item))
   doc->deleteEntry(QString(item->text(0))+QString("@")+QString(item->text(1)));
  item=item->nextSibling();
 }
 */
 doc->deleteEntry(QString(saveItem->text(0))+QString("@")+QString(saveItem->text(1)));
 updateListView();

 if (charDB_ListView->firstChild()!=NULL)
  charDB_ListView->setSelected(charDB_ListView->firstChild(),true);

 setDialogState();
}


void DlgConnectionWizard::applyInputTo(QListViewItem* item)
{

 CharDBEntry* entry;
 CCharacterProfile* prof;
 CMudProfile* mud;
 QString s;

 if (item==NULL) return; //ups?

 entry=doc->getEntry(QString(item->text(0)),QString(item->text(1)));

 if (entry==NULL) return; // ups?

 entry->ID=login_LineEdit->text()+QString("@")+mudTitle_LineEdit->text();

 entry->login=login_LineEdit->text();

 prof=doc->getCharacterProfile(entry->proffile);

 if (QString(charProfileShow_Label->text())=="")
 {
  s=entry->ID;
  s=(s.simplifyWhiteSpace()).lower();
  s.replace(QRegExp(" "),"");
  s=s+QString(".prof");
 }
 else
 {
  s=charProfileShow_Label->text();
  s=(s.simplifyWhiteSpace()).lower();
  s.replace(QRegExp(" "),"");
  s=s+QString(".prof");
 }

// if (s!=prof->getFileName())
// {
//  s=QFileDialog::getSaveFileName(QDir::homeDirPath()+"/.kde/share/apps/kmud/profiles/"+s,"*.prof",this);
// }


 // if we changed the filename to a profile that already exists -> use that
 if ((doc->getCharacterProfile(s)!=NULL) && (prof->getFileName()!=s))
 {
//  doc->deleteCharacterProfile(prof);
  prof=doc->getCharacterProfile(s);
 } else prof->setFileName(s); // else set new file name

 prof->setFileName(s);
 entry->proffile=prof->getFileName();


 mud=doc->getMudProfile(entry->mudfile);

 s=mudTitle_LineEdit->text();
 s=(s.simplifyWhiteSpace()).lower();
 s.replace(QRegExp(" "),"");
 s=s+QString(".mud");
// if (s!=prof->getFileName())
// {
//  s=QFileDialog::getSaveFileName(QDir::homeDirPath()+"/.kde/share/apps/kmud/muds/"+s,"*.mud",this);
// }

 // if we changed the filename to a profile that already exists -> use that
 if ((doc->getMudProfile(s)!=NULL) && (mud->getFileName()!=s))
 {
//  doc->deleteMudProfile(mud);
  mud=doc->getMudProfile(s);
 } else mud->setFileName(s);  // else set new file name

 entry->mudfile=mud->getFileName();

 mud->setTitle(mudTitle_LineEdit->text());
 mud->setHostName(mudHost_LineEdit->text());
 s=mudPort_LineEdit->text();
 mud->setPort(s.toInt());

 // change the ListViewItem to new values
 item->setText(0,login_LineEdit->text());
 item->setText(1,mudTitle_LineEdit->text());

}



void DlgConnectionWizard::slotSelectMudPressed()
{
 DlgSelectMud d(doc,this,"SelectMudDialog");

 CMudProfile* mud=NULL;
 QString s;

 if (d.exec())
 {
  mud=d.getSelectedMud();
  if (mud!=NULL)
  {
   mudTitle_LineEdit->setText(mud->getTitle());
   mudHost_LineEdit->setText(mud->getHostName());
   s.setNum(mud->getPort());
   mudPort_LineEdit->setText(s);
  }
 }

}

void DlgConnectionWizard::slotSelectProfilePressed()
{
 DlgSelectProfile d(doc,this,"SelectProfileDialog");
 CCharacterProfile* prof;
 QFileInfo fi;

 if (d.exec())
 {
  prof=d.getSelectedFileName();
  fi.setFile("",prof->getFileName());
  charProfileShow_Label->setText(fi.baseName());
 }
}


void DlgConnectionWizard::updateListView()
{
 QListViewItem* item;
 CharDBEntry* entry;
 CMudProfile* mud;
 QString s1,s2;


 if (saveItem)     // preserve saveItem
 {
  s1=saveItem->text(0); s2=saveItem->text(1);

  charDB_ListView->clear();

  entry=doc->first();
  while (entry!=NULL)
  {
   mud = doc->getMudProfile(entry->mudfile);
   item = new QListViewItem(charDB_ListView, entry->login, mud->getTitle());
   entry=doc->next();
  }

  saveItem=NULL;
  item=charDB_ListView->firstChild();
  while (item!=NULL)
  {
   if ( (QString(item->text(0))==s1) && (QString(item->text(1))==s2) )
    {
     saveItem=item;
     charDB_ListView->setSelected(saveItem,true);
    }
   item=item->nextSibling();
  }

 }

 else
 {
  charDB_ListView->clear();

  entry=doc->first();
  while (entry!=NULL)
  {
   mud = doc->getMudProfile(entry->mudfile);
   item = new QListViewItem(charDB_ListView, entry->login, mud->getTitle());
   entry=doc->next();
  }
 }

}


void DlgConnectionWizard::slotShowItem(QListViewItem* i)
{

 CharDBEntry* entry;
 CMudProfile* mud;
 CCharacterProfile* prof;
 QString s;
 QFileInfo fi;

 applyInputTo(saveItem);

 entry=doc->getEntry(i->text(0),i->text(1));

 if (entry!=NULL)
 {
  login_LineEdit->setText(entry->login);
  mud=doc->getMudProfile(entry->mudfile);
  prof=doc->getCharacterProfile(entry->proffile);
  if (mud!=NULL)
  {
   fi.setFile("",prof->getFileName());
   charProfileShow_Label->setText(fi.baseName());
   mudTitle_LineEdit->setText(mud->getTitle());
   mudHost_LineEdit->setText(mud->getHostName());
   s.setNum(mud->getPort());
   mudPort_LineEdit->setText(s);
  }
 }

 saveItem=i;

 setDialogState();
}

QString DlgConnectionWizard::getLogin()
{
 return login_LineEdit->text();
}

QString DlgConnectionWizard::getMudTitle()
{
 return mudTitle_LineEdit->text();
}

QString DlgConnectionWizard::getMudHost()
{
 return mudHost_LineEdit->text();
}

QString DlgConnectionWizard::getMudPort()
{
 return mudPort_LineEdit->text();
}

void DlgConnectionWizard::slotChangedLogin(const char* s)
{
 if (saveItem) saveItem->setText(0,s);
}

void DlgConnectionWizard::slotChangedMudTitle(const char* s)
{
 if (saveItem) saveItem->setText(1,s);
}

void DlgConnectionWizard::setDialogState()
{

 if (charDB_ListView->currentItem()==NULL)
 {
  charProfileSelect_Button->setEnabled(false);
  mudListSelect_Button->setEnabled(false);
  login_LineEdit->setEnabled(false);
  login_LineEdit->setText("");
  mudTitle_LineEdit->setEnabled(false);
  mudTitle_LineEdit->setText("");
  mudHost_LineEdit->setEnabled(false);
  mudHost_LineEdit->setText("");
  mudPort_LineEdit->setEnabled(false);
  mudPort_LineEdit->setText("");
  charProfileShow_Label->setText("");
 }
 else
 {
  charProfileSelect_Button->setEnabled(true);
  mudListSelect_Button->setEnabled(true);
  login_LineEdit->setEnabled(true);
  mudTitle_LineEdit->setEnabled(true);
  mudHost_LineEdit->setEnabled(true);
  mudPort_LineEdit->setEnabled(true);
 }
}

QString DlgConnectionWizard::getCharacterID()
{
 CharDBEntry* entry;

 entry=doc->getEntry(QString(saveItem->text(0)),QString(saveItem->text(1)));
 return entry->proffile;
}


QString DlgConnectionWizard::getMudID()
{
 CharDBEntry* entry;

 entry=doc->getEntry(QString(saveItem->text(0)),QString(saveItem->text(1)));
 return entry->mudfile;
}
