/***************************************************************************
                          dlgoptionpreferences.cpp
                            -------------------
    begin                : Sat Sep 11 1999
    copyright            : (C) 1999 by Stephan Uhlmann
    email                : suhlmann@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kapp.h>

#include "dlgoptionpreferences.h"


DlgOptionPreferences::DlgOptionPreferences(QWidget *parent, const char *name)
 : QTabDialog(parent,name, true)
{

setCaption(i18n("Preferences"));

generalTab = new DlgOptionGeneral(this,"General");
echoTab = new DlgOptionEcho(this,"Echo");
memoryTab = new DlgOptionMemory(this,"Memory");

addTab(generalTab,i18n("General"));
addTab(echoTab,i18n("Echo"));
addTab(memoryTab,i18n("Buffers"));

setOkButton(i18n("OK"));
setDefaultButton(i18n("Default"));
setCancelButton(i18n("Cancel"));

connect(this, SIGNAL(applyButtonPressed()), SLOT(slotOKPressed()));
connect(this, SIGNAL(defaultButtonPressed()), SLOT(slotDefaultPressed()));

this->resize(440,420);

slotDefaultPressed();

}


DlgOptionPreferences::~DlgOptionPreferences(){

	// no need to delete widgets that are children of this class
	// since they will be destroyed when destroying the parent
	// see: ~QObject()
}

void DlgOptionPreferences::slotOKPressed(){

}

void DlgOptionPreferences::slotDefaultPressed(){
setKeepLastInput(false);
setAutoReconnect(true);
setLocalEchoMode(2);
setInputEchoMode(2);
setOutputSize(5000);
setInputSize(1000);
setSeparator(";");
}



int DlgOptionPreferences::getLocalEchoMode()
{
 return echoTab->getLocalEchoMode();
}

int DlgOptionPreferences::getInputEchoMode()
{
 return echoTab->getInputEchoMode();
}

int DlgOptionPreferences::getInputSize()
{
return memoryTab->getInputSize();
}

int DlgOptionPreferences::getOutputSize()
{
return memoryTab->getOutputSize();
}

void DlgOptionPreferences::setLocalEchoMode(int i)
{
 echoTab->setLocalEchoMode(i);
}

void DlgOptionPreferences::setInputEchoMode(int i)
{
 echoTab->setInputEchoMode(i);
}


void DlgOptionPreferences::setInputSize(int i)
{
 memoryTab->setInputSize(i);
}

void DlgOptionPreferences::setOutputSize(int i)
{
 memoryTab->setOutputSize(i);
}

bool DlgOptionPreferences::getKeepLastInput()
{
 return generalTab->getKeepLastInput();
}


void DlgOptionPreferences::setKeepLastInput(bool b)
{
 generalTab->setKeepLastInput(b);
}

bool DlgOptionPreferences::getAutoReconnect()
{
 return generalTab->getAutoReconnect();
}


void DlgOptionPreferences::setAutoReconnect(bool b)
{
 generalTab->setAutoReconnect(b);
}

QString DlgOptionPreferences::getSeparator()
{
    return generalTab->getSeparator();
}

void DlgOptionPreferences::setSeparator(QString s)
{
    generalTab->setSeparator(s);
}

void DlgOptionPreferences::setUseBeep(bool b) {
    generalTab->setUseBeep(b);
}

bool DlgOptionPreferences::getUseBeep() {
    return generalTab->getUseBeep();
}