/***************************************************************************
                          dlgprofilealiases.cpp
                      -------------------
    description          :
    begin                : Mon Nov 1 1999                                           
    copyright            : (C) 1999 by Stephan Uhlmann                         
    email                : suhlmann@gmx.de                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include "dlgprofilealiases.h"
#include <qkeycode.h>

DlgProfileAliases::DlgProfileAliases(KmudDoc* document, QWidget *parent, const char *name) : QDialog(parent,name,true){
	profile = document->getCharacterProfile(document->getCurrentCharacterID());
	profile->readAliases();
	
	initDialog();
	
	ProfileName_Label->setText(document->getCurrentCharacterID());

    aliases_ListView->addColumn( i18n("Alias Name") );
    aliases_ListView->addColumn( i18n("Macro") );
    aliases_ListView->addColumn( i18n("Definition") );
    aliases_ListView->setAllColumnsShowFocus( true );
    aliases_ListView->setColumnWidth(0, 80);
    aliases_ListView->setColumnWidth(1, 45);
    aliases_ListView->setColumnWidth(2, 299);
    aliases_ListView->setColumnWidthMode(0, QListView::Manual);
    aliases_ListView->setColumnWidthMode(1, QListView::Manual);
    aliases_ListView->setColumnWidthMode(2, QListView::Manual);


	connect( cancel_Button, SIGNAL(clicked()), SLOT(slotCancelPressed()) );
	connect( ok_Button, SIGNAL(clicked()), SLOT(slotOKPressed()) );
	connect( new_Button, SIGNAL(clicked()), SLOT(slotNewPressed()) );
	connect( delete_Button, SIGNAL(clicked()), SLOT(slotDeletePressed()) );
	connect( aliases_ListView, SIGNAL(selectionChanged(QListViewItem*)), SLOT(slotShowItem(QListViewItem*)) );
    connect( change_Button, SIGNAL(clicked()), SLOT(slotChangePressed()) );
    connect( apply_Button, SIGNAL(clicked()), SLOT(slotApplyPressed()) );
    		
	if (profile == NULL) {
	    aliases_ListView->clear();
	    QListViewItem a = new QListViewItem(aliases_ListView, "", "", QString("NO ACTIVE PROFILE"));
	    return;
	}
	updateListView();
	
	
	if (aliases_ListView->firstChild() != NULL) {
	    aliases_ListView->setSelected(aliases_ListView->firstChild(), true);
	}
}

DlgProfileAliases::~DlgProfileAliases(){
}

/** Updates the List View of all the aliases */
void DlgProfileAliases::updateListView() {
    QListViewItem* item;
    AliasEntry* alias;

    aliases_ListView->clear();
    alias = profile->firstAlias();

    while (alias != NULL) {
        item = new QListViewItem(aliases_ListView, alias->name, translateKey(alias->macro), alias->definition);
        alias = profile->nextAlias();
    }
}

/**                 **
 **    S L O T S    **
 **                 **/

/** When "OK" is pressed */
void DlgProfileAliases::slotOKPressed(){
    profile->writeAliases();
    accept();
}

/** When "CANCEL" is pressed */
void DlgProfileAliases::slotCancelPressed(){
    reject();
}

/** When "NEW" is pressed */
void DlgProfileAliases::slotNewPressed(){
    profile->insertAlias("", "", 0);
    updateListView();

    if (aliases_ListView->firstChild() != NULL) {
        aliases_ListView->setSelected(aliases_ListView->firstChild(), true);
    }
}

/** When "DELETE" is pressed */
void DlgProfileAliases::slotDeletePressed(){
    QListViewItem* item;

    item = aliases_ListView->firstChild();

    while (item != NULL) {
        if (aliases_ListView->isSelected(item))
            profile->removeAlias(QString(item->text(0)));
        item = item->nextSibling();
    }
    updateListView();

    if (aliases_ListView->firstChild() != NULL) {
        aliases_ListView->setSelected(aliases_ListView->firstChild(), true);
    }
    else {
        aliasName_LineEdit->setText("");
        aliasDefinition_LineEdit->setText("");
        macro_Label->setText(translateKey(0));
    }
}

/** applys the current changes */
/*
void DlgProfileAliases::apply(QListViewItem* item){
    AliasEntry* alias;
    QString n, d;

    if (item == NULL) return;

    n = aliasName_LineEdit->text();
    d = aliasDefinition_LineEdit->text();

    alias = profile->getAlias(QString(item->text(0)));
    if (alias == NULL) return;  // nothing was selected

    alias->name = n;
    alias->definition = d;

    updateListView();

    item = aliases_ListView->firstChild();
    while (item != NULL) {
        if ((QString(item->text(0)) == QString(n)) && (QString(item->text(2)) == QString(d)) ) {
            aliases_ListView->setSelected(item, true);
        }
        item = item->nextSibling();
    }
}
*/

void DlgProfileAliases::slotShowItem(QListViewItem* item){
    AliasEntry* a;
    QString n, d;

    a = profile->getAlias(QString(item->text(0)));
    if (a == NULL) return;

    n = item->text(0);
    d = item->text(2);

    if ((n != NULL) && (d != NULL))
    {
        aliasName_LineEdit->setText(n);
        aliasDefinition_LineEdit->setText(d);
        macro_Label->setText(translateKey(a->macro));
    }
    else
    {
        aliasName_LineEdit->setText("");
        aliasDefinition_LineEdit->setText("");
        macro_Label->setText("0");
    }
}

/** When "APPLY" is pressed */
void DlgProfileAliases::slotApplyPressed(){
    QListViewItem* item;
    AliasEntry* alias;
    QString n, d;

    n = aliasName_LineEdit->text();
    d = aliasDefinition_LineEdit->text();

    item = aliases_ListView->firstChild();
    alias = NULL;
    while (item != NULL) {
        if (n == "") {
            QMessageBox::information(this, "Warning", "You must specify an alias name.");
            aliasName_LineEdit->setFocus();
            return;
        } else if (n == item->text(0)) {
            QMessageBox::information(this, "Warning", "That alias name already exists.");
            aliasName_LineEdit->setFocus();
            return;
        }
        item = item->nextSibling();
    }

    item = aliases_ListView->firstChild();
    alias = NULL;
    while (item != NULL) {
        if (aliases_ListView->isSelected(item))
            alias = profile->getAlias(QString(item->text(0)));
        item = item->nextSibling();
    }

    if (alias == NULL) return;  // nothing was selected

    alias->name = n;
    alias->definition = d;

    updateListView();

    item = aliases_ListView->firstChild();
    while (item != NULL) {
        if ((QString(item->text(0)) == QString(n)) && (QString(item->text(2)) == QString(d)) )
            aliases_ListView->setSelected(item, true);
        item = item->nextSibling();
    }
}

void DlgProfileAliases::slotChangePressed() {
    AliasEntry* a;
    QListViewItem* item;
    QString n, d;

    item = aliases_ListView->firstChild();
    a = NULL;
    while (item != NULL) {
        if (aliases_ListView->isSelected(item))
            a = profile->getAlias(QString(item->text(0)));
        item = item->nextSibling();
    }

    if (a == NULL) return;

    n = a->name;
    d = a->definition;
    DlgProfileMacros dlg(profile, a, this, "Macros");

    if (dlg.exec())
    {
    }

    slotShowItem(aliases_ListView->currentItem());
    updateListView();

    item = aliases_ListView->firstChild();
    while (item != NULL) {
        if ((QString(item->text(0)) == QString(n)) && (QString(item->text(2)) == QString(d)) ) {
            aliases_ListView->setSelected(item, true);
        }
        item = item->nextSibling();
    }

}

QString DlgProfileAliases::translateKey(int key)
{
    if (key > 0) {
        switch (key) {
            case Key_F2:
                return QString("F2");
                break;
            case Key_F3:
                return QString("F3");
                break;
            case Key_F4:
                return QString("F4");
                break;
            case Key_F5:
                return QString("F5");
                break;
            case Key_F6:
                return QString("F6");
                break;
            case Key_F7:
                return QString("F7");
                break;
            case Key_F8:
                return QString("F8");
                break;
            case Key_F9:
                return QString("F9");
                break;
            case Key_F10:
                return QString("F10");
                break;
            case Key_F11:
                return QString("F11");
                break;
            case Key_F12:
                return QString("F12");
                break;
            default:
                return QString("ERROR");
                break;
        }
    }
    else
        return QString("None");
}
