/***************************************************************************
                          dlgprofiletriggers.cpp
                      -------------------
    description          :
    begin                : Tue Nov 2 1999                                           
    copyright            : (C) 1999 by Stephan Uhlmann                         
    email                : suhlmann@gmx.de                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include "dlgprofiletriggers.h"

DlgProfileTriggers::DlgProfileTriggers(KmudDoc* document, QWidget *parent, const char *name) : QDialog(parent,name,true){

	profile = document->getCharacterProfile(document->getCurrentCharacterID());
	profile->readTriggers();
	
	initDialog();
	
	triggerProfile_Label->setText(document->getCurrentCharacterID());

	connect( cancel_Button, SIGNAL(clicked()), SLOT(slotCancelPressed()) );
	connect( ok_Button, SIGNAL(clicked()), SLOT(slotOKPressed()) );
	connect( new_Button, SIGNAL(clicked()), SLOT(slotNewPressed()) );
	connect( delete_Button, SIGNAL(clicked()), SLOT(slotDeletePressed()) );
	connect( apply_Button, SIGNAL(clicked()), SLOT(slotApplyPressed()) );
	connect( triggers_ListBox, SIGNAL(highlighted(int)), SLOT(slotShowItem(int)) );
	
	if (profile == NULL) {
	    triggers_ListBox->clear();
	    triggers_ListBox->insertItem("NO ACTIVE PROFILE", 0);
	    return;
	}
	updateListView();
	
    if (triggers_ListBox->text(0) != NULL)
        triggers_ListBox->setSelected(0, true);
}

DlgProfileTriggers::~DlgProfileTriggers(){
}

/** Updates the List View of all the triggers */
void DlgProfileTriggers::updateListView() {
    TriggerEntry* trigger;

    triggers_ListBox->clear();

    trigger = profile->firstTrigger();

    while (trigger != NULL) {
        triggers_ListBox->insertItem(trigger->name, 0);
        trigger = profile->nextTrigger();
    }
}

/**                 **
 **    S L O T S    **
 **                 **/

/** When "OK" is pressed */
void DlgProfileTriggers::slotOKPressed(){
    profile->writeTriggers();
    accept();
}

/** When "CANCEL" is pressed */
void DlgProfileTriggers::slotCancelPressed(){
    reject();
}

/** When "NEW" is pressed */
void DlgProfileTriggers::slotNewPressed(){
    profile->insertTrigger("", "");

    updateListView();

    if (triggers_ListBox->text(0) != NULL)
        triggers_ListBox->setSelected(0, true);
}

/** When "DELETE" is pressed */
void DlgProfileTriggers::slotDeletePressed(){
    uint i = 0;

    while (i< triggers_ListBox->count()) {
        if (triggers_ListBox->isSelected(i)) {
            profile->removeTrigger(QString(triggers_ListBox->text(i)));
        }
        i++;
    }
    updateListView();

    if (triggers_ListBox->text(0) != NULL)
        triggers_ListBox->setSelected(0, true);
}

/** When "APPLY" is pressed */
void DlgProfileTriggers::slotApplyPressed(){
    TriggerEntry* trig;
    QString n, d;
    uint i = 0;
    int current = 0;

    n = triggerName_LineEdit->text();
    d = triggerDefinition_LineEdit->text();

    trig = NULL;
    while (i < triggers_ListBox->count()) {
        if (triggers_ListBox->isSelected(i)) {
            trig = profile->getTrigger(QString(triggers_ListBox->text(i)));
        }
        i++;
    }

    if (trig == NULL) return;  // nothing was selected

    trig->name = n;
    trig->definition = d;
    current = triggers_ListBox->currentItem();

    triggers_ListBox->removeItem(current);
    triggers_ListBox->insertItem(n, current);

    triggers_ListBox->setSelected(current, true);
}

void DlgProfileTriggers::slotShowItem(int i){
    TriggerEntry* trigger;
    QString name = QString(triggers_ListBox->text(i));

//    n = QString(triggers_ListBox->text(triggers_ListBox->currentItem()));
    trigger = profile->getTrigger(name);

    if (name != NULL)
    {
        triggerName_LineEdit->setText(name);
        triggerDefinition_LineEdit->setText(trigger->definition);
    }
}