/***************************************************************************
                          kmud.cpp
                      -------------------
    description          : Main Application Class
    begin                : Sun Jun 20 14:17:54 MEST 1999
    copyright            : (C) 1999 by Stephan Uhlmann
    email                : suhlmann@gmx.de                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include "kmud.h"


KmudApp::KmudApp()
{
  setCaption("Kmud " VERSION);


  ///////////////////////////////////////////////////////////////////
  // read the config file options
  // read them _before_ the inits so we init the right
  readOptions();

  this->resize(mainWindowWidth,mainWindowHeight);

  autoReconnect = false;

  doc = new KmudDoc(this);
  doc->setCurrentCharacterID("");
  doc->setCurrentMudID("");

  currentLogin="";
  currentMudTitle="";


  ///////////////////////////////////////////////////////////////////
  // call inits to invoke all other construction parts
  initMenuBar();
  initToolBar();
  initStatusBar();
  initView();
  initTimer();


  ///////////////////////////////////////////////////////////////////
  // create new Telnet
  mud = new CTelnet;

  ///////////////////////////////////////////////////////////////////
  // enable bars dependend on config file setups
  if (!bViewToolbar)
    enableToolBar(KToolBar::Hide,0);
  if (!bViewStatusbar)
    enableStatusBar(KStatusBar::Hide);



  menuBar()->setMenuBarPos(menu_bar_pos);
  toolBar()->setBarPos(tool_bar_pos);
}

KmudApp::~KmudApp()
{
	// Make sure that is disable as the applications is about to quit
	autoReconnect = false;
	delete mud;
	delete mud_menu;
	delete view_menu;
	delete option_menu;
	delete profile_menu;
	delete connectTimer;
	delete connectIcon;
}

void KmudApp::enableCommand(int id_)
{
  ///////////////////////////////////////////////////////////////////
  // enable menu and toolbar functions by their ID's
  menuBar()->setItemEnabled(id_,true);
  toolBar()->setItemEnabled(id_,true);
}

void KmudApp::disableCommand(int id_)
{
  ///////////////////////////////////////////////////////////////////
  // disable menu and toolbar functions by their ID's
  menuBar()->setItemEnabled(id_,false);
  toolBar()->setItemEnabled(id_,false);
}

void KmudApp::initTimer()
{
	//////////////////////////////////////////////////////////////////
	// setup the connection timer
	connectTimer = new QTimer( this );
	connect( connectTimer, SIGNAL(timeout()),this, SLOT(slotConnectionTime()) );
	connectHour = 0;
	connectMin = 0;
	connectSec = 0;
}

void KmudApp::initMenuBar()
{

  ///////////////////////////////////////////////////////////////////
  // MENUBAR  

  ///////////////////////////////////////////////////////////////////
  // menuBar entry mud_menu
  mud_menu = new QPopupMenu();
  mud_menu->insertItem(Icon("mini/kmud.xpm"), i18n("&New Window"), ID_MUD_NEW_WINDOW );
  mud_menu->insertSeparator();
  mud_menu->insertItem(Icon("mini/kmud_connectionwizard.xpm"), i18n("&Connection Wizard"), ID_MUD_CONNECTION_WIZARD );
  mud_menu->insertItem(Icon("mini/kmud_quickconnect.xpm"), i18n("&Quick Connect"), ID_MUD_CONNECT );
  mud_menu->insertItem(Icon("mini/kmud_disconnect.xpm"), i18n("Close"), ID_MUD_CLOSE );
  mud_menu->insertSeparator();
  mud_menu->insertItem(i18n("C&lose Window"), ID_MUD_CLOSE_WINDOW);
  mud_menu->insertSeparator();
  mud_menu->insertItem(i18n("E&xit"), ID_MUD_QUIT );

  // mud_menu key accelerators
  mud_menu->setAccel(CTRL+Key_N, ID_MUD_NEW_WINDOW);
  mud_menu->setAccel(CTRL+Key_O, ID_MUD_CONNECTION_WIZARD);
  mud_menu->setAccel(CTRL+Key_E, ID_MUD_CONNECT);
  mud_menu->setAccel(CTRL+Key_Q, ID_MUD_QUIT);
  mud_menu->setAccel(ALT+Key_X, ID_MUD_QUIT);

  ///////////////////////////////////////////////////////////////////
  // menuBar entry view_menu
  view_menu = new QPopupMenu();
  view_menu->setCheckable(true);
  view_menu->insertItem(i18n("Tool&bar"), ID_VIEW_TOOLBAR);
  view_menu->insertItem(i18n("&Statusbar"), ID_VIEW_STATUSBAR );
  view_menu->insertItem(i18n("&Alternative Input "), ID_VIEW_ALTINPUT );

  view_menu->setItemChecked(ID_VIEW_TOOLBAR, bViewToolbar);
  view_menu->setItemChecked(ID_VIEW_STATUSBAR, bViewStatusbar);
  view_menu->setItemChecked(ID_VIEW_ALTINPUT, bViewAltInput);
//  view_menu->insertItem(i18n("Auto &Mapper"),ID_VIEW_MAPPER);



  ///////////////////////////////////////////////////////////////////
  // menuBar entry option_menu
  option_menu = new QPopupMenu();
  option_menu->setCheckable(true);
  option_menu->insertItem(i18n("&Fonts..."), ID_OPTION_FONT);
  option_menu->insertItem(i18n("&Colors..."), ID_OPTION_COLORS);
  option_menu->insertItem(i18n("&Preferences..."), ID_OPTION_PREFERENCES);
  option_menu->insertSeparator();
  mudOptionsID = option_menu->insertItem(i18n("Current &MUD Settings..."), ID_OPTION_MUD);
  option_menu->insertSeparator();
  option_menu->insertItem(i18n("&Save Options"), ID_OPTION_SAVE);

  if (doc->getCurrentMudID() == "") {
    option_menu->setItemEnabled(mudOptionsID, false);
  } else {
    option_menu->setItemEnabled(mudOptionsID, true);
  }

  ///////////////////////////////////////////////////////////////////
  // menuBar entry profile_menu
  profile_menu = new QPopupMenu();
  profile_menu->setCheckable(true);
  aliasID = profile_menu->insertItem(Icon("mini/kmud_aliases.xpm"), i18n("&Aliases..."), ID_PROFILE_ALIASES);
//  triggerID = profile_menu->insertItem(Icon("mini/kmud_triggers.xpm"), i18n("&Triggers..."), ID_PROFILE_TRIGGERS);
  profile_menu->insertSeparator();
  saveID = profile_menu->insertItem(i18n("&Save Current Profile"), ID_PROFILE_SAVE);
  if (doc->getCurrentCharacterID() == "") {
      profile_menu->setItemEnabled(aliasID, false);
      profile_menu->setItemEnabled(triggerID, false);
      profile_menu->setItemEnabled(saveID, false);
  } else {
      profile_menu->setItemEnabled(aliasID, true);
      profile_menu->setItemEnabled(triggerID, true);
      profile_menu->setItemEnabled(saveID, true);
  }

  ///////////////////////////////////////////////////////////////////
  // menuBar entry help_menu
  help_menu = new QPopupMenu();
  QString aboutString="";
  aboutString+=i18n("Kmud Version ");
  aboutString+=VERSION;
  aboutString+=i18n("\nCopyright (c) 1999 The Kmud Team <kmud-devel@barney.cs.uni-potsdam.de>\n    Stephan Uhlmann <suhlmann@gmx.de>\n    John-Paul Stanford <john-paul.stanford@virgin.net>\n    Luke Ehresman <luke@usa.om.org>\n\nKmud Homepage: http://fara.cs.uni-potsdam.de/~uhlmann/kmud/\n\nKmud is distributed under the GNU General Public License.\nSee the file COPYING for further detail.");
  help_menu = kapp->getHelpMenu(true, aboutString);


  ///////////////////////////////////////////////////////////////////
  // MENUBAR CONFIGURATION
  // set menuBar() the current menuBar and the position due to config file
  menuBar()->insertItem(i18n("&Mud"), mud_menu);
  menuBar()->insertItem(i18n("&View"), view_menu);
  menuBar()->insertItem(i18n("&Options"), option_menu);
  menuBar()->insertItem(i18n("&Profile"), profile_menu);

  menuBar()->insertSeparator();
  menuBar()->insertItem(i18n("&Help"), help_menu);

  ///////////////////////////////////////////////////////////////////
  // CONNECT THE SUBMENU SLOTS WITH SIGNALS

  CONNECT_CMD(mud_menu);
  CONNECT_CMD(view_menu);
  CONNECT_CMD(option_menu);
  CONNECT_CMD(profile_menu);

}
void KmudApp::initToolBar()
{

  ///////////////////////////////////////////////////////////////////
  // TOOLBAR
  // set toolBar() the current toolBar and the position due to config file
  toolBarConnectButtonPosition=toolBar()->insertButton(Icon("mini/kmud_connectionwizard.xpm"), ID_MUD_CONNECTION_WIZARD, true, i18n("Connection Wizard"));
  toolBar()->insertSeparator();
  aliasToolID = toolBar()->insertButton(Icon("mini/kmud_aliases.xpm"), ID_PROFILE_ALIASES, true, i18n("Aliases"));
//  triggerToolID = toolBar()->insertButton(Icon("mini/kmud_triggers.xpm"), ID_PROFILE_TRIGGERS, true, i18n("Triggers"));
  toolBar()->insertSeparator();
  toolBar()->insertButton(Icon("help.xpm"), ID_HELP, SIGNAL(pressed()), kapp, SLOT(appHelpActivated()), true, i18n("Help"));
  toolBar()->setItemEnabled(aliasID, false);
  toolBar()->setItemEnabled(triggerID, false);

  ///////////////////////////////////////////////////////////////////
  // INSERT YOUR APPLICATION SPECIFIC TOOLBARS HERE -e.g. tool_bar_1:
  // add functionality for new created toolbars in:
  // enableCommand, disableCommand, in the menuBar() and an additional function slotViewToolbar_1
  // for that also create a bViewToolbar_1 and a KConfig entry (see Constructor).
  // Also update resource values and commands 


  ///////////////////////////////////////////////////////////////////
  // CONNECT THE TOOLBAR SLOTS WITH SIGNALS - add new created toolbars
  CONNECT_TOOLBAR(toolBar());

}

void KmudApp::initStatusBar()
{
  	///////////////////////////////////////////////////////////////////
  	//STATUSBAR
	connectIcon = new QLabel(statusBar());
	connectIcon->setPixmap(Icon("mini/mini-connect.xpm"));
	connectIcon->setFrameStyle(QFrame::Panel | QFrame::Sunken);
  	statusBar()->setInsertOrder(KStatusBar::RightToLeft);
        statusBar()->insertItem(i18n("00:00:00"), ID_STATUS_TIME );
        statusBar()->insertWidget(connectIcon,25,ID_STATUS_ICON); 
	statusBar()->insertItem(i18n("Ready."), ID_STATUS_MSG );
}

void KmudApp::initView()
{ 
  ////////////////////////////////////////////////////////////////////
  // set the main widget here
  KApplication *app=KApplication::getKApplication();
  view = new KmudView(app,doc,this);
  setView(view);

  ///////////////////////////////////////////////////////////////////
  // set view options as defined in readOptions()
  view->setOutputForegroundColor(mudForegroundColor);
  view->setOutputBackgroundColor(mudBackgroundColor);
  view->setOutputFont(mudFont);
  view->setInputFont(inputFont);
  view->setInputBackgroundColor(mudBackgroundColor);
  view->setLocalEchoColor(mudLocalEchoColor);

  view->setDebugHighlightColor(mudDebugHighlightColor);

  view->setOutputANSIBlack(mudANSIBlack);
  view->setOutputANSIDarkRed(mudANSIDarkRed);
  view->setOutputANSIDarkGreen(mudANSIDarkGreen);
  view->setOutputANSIDarkYellow(mudANSIDarkYellow);
  view->setOutputANSIDarkBlue(mudANSIDarkBlue);
  view->setOutputANSIDarkMagenta(mudANSIDarkMagenta);
  view->setOutputANSIDarkCyan(mudANSIDarkCyan);
  view->setOutputANSIDarkWhite(mudANSIDarkWhite);

  view->setOutputANSIGray(mudANSIGray);
  view->setOutputANSILightRed(mudANSILightRed);
  view->setOutputANSILightGreen(mudANSILightGreen);
  view->setOutputANSILightYellow(mudANSILightYellow);
  view->setOutputANSILightBlue(mudANSILightBlue);
  view->setOutputANSILightMagenta(mudANSILightMagenta);
  view->setOutputANSILightCyan(mudANSILightCyan);
  view->setOutputANSILightWhite(mudANSILightWhite);

  view->setOutputBgBlack(mudBgBlack);
  view->setOutputBgRed(mudBgRed);
  view->setOutputBgGreen(mudBgGreen);
  view->setOutputBgYellow(mudBgYellow);
  view->setOutputBgBlue(mudBgBlue);
  view->setOutputBgMagenta(mudBgMagenta);
  view->setOutputBgCyan(mudBgCyan);
  view->setOutputBgWhite(mudBgWhite);

	view->setAltInput(bViewAltInput);
	view->setMaxHistoryLines(maxInputLines);
	view->setMaxMudViewLines(maxOutputLines);
	view->setSeparator(separator);
	view->setUseBeep(useBeep);

	view->setLocalEchoMode(localEchoMode);
	view->setInputEchoMode(inputEchoMode);

  view->setKeepLastInput(keepLastInput);
}


void KmudApp::saveOptions()
{
  KConfig *config = kapp->getConfig();

  config->setGroup("APPEARANCE");
  config->writeEntry("WindowWidth",this->width());
  config->writeEntry("WindowHeight",this->height());
  config->writeEntry("ShowToolbar",toolBar()->isVisible());
  config->writeEntry("ShowStatusbar",statusBar()->isVisible());
  config->writeEntry("ShowAltInput",bViewAltInput);
  config->writeEntry("MenuBarPos", (int)menuBar()->menuBarPos());
  config->writeEntry("ToolBar_Pos", (int)toolBar()->barPos());

  config->setGroup("OPTIONS");

  QColor fgc=view->getOutputForegroundColor();
  config->writeEntry("MudTextColor",fgc);

  QColor bgc=view->getOutputBackgroundColor();
  config->writeEntry("MudBackgroundColor",bgc);

  QFont fo=view->getOutputFont();
  config->writeEntry("MudFont",fo);

  QFont fi=view->getInputFont();
  config->writeEntry("InputFont",fi);

  QColor lec=view->getLocalEchoColor();
  config->writeEntry("MudLocalEchoColor",lec);

  QColor dhc=view->getDebugHighlightColor();
  config->writeEntry("MudDebugHighlightColor", dhc);

  config->writeEntry("MaxOutputLines",maxOutputLines);
  config->writeEntry("MaxInputLines",maxInputLines);
  config->writeEntry("Local_Echo",localEchoMode);
  config->writeEntry("Input_Echo",inputEchoMode);
  config->writeEntry("Keep_Last_input",keepLastInput);
	config->writeEntry("AutoReconnect",autoReconnectOption);
	config->writeEntry("CharSeparator",separator);
	config->writeEntry("UseBeep",useBeep);

  config->setGroup("HISTORY");
	if (lastConnectHost.at(0)!=NULL) config->writeEntry("0Host",lastConnectHost.at(0)); else config->writeEntry("0Host","");
	if (lastConnectPort.at(0)!=NULL) config->writeEntry("0Port",lastConnectPort.at(0)); else config->writeEntry("0Port","");
	if (lastConnectHost.at(1)!=NULL) config->writeEntry("1Host",lastConnectHost.at(1)); else config->writeEntry("1Host","");
	if (lastConnectPort.at(1)!=NULL) config->writeEntry("1Port",lastConnectPort.at(1)); else config->writeEntry("1Port","");
	if (lastConnectHost.at(2)!=NULL) config->writeEntry("2Host",lastConnectHost.at(2)); else config->writeEntry("2Host","");
	if (lastConnectPort.at(2)!=NULL) config->writeEntry("2Port",lastConnectPort.at(2)); else config->writeEntry("2Port","");
	if (lastConnectHost.at(3)!=NULL) config->writeEntry("3Host",lastConnectHost.at(3)); else config->writeEntry("3Host","");
	if (lastConnectPort.at(3)!=NULL) config->writeEntry("3Port",lastConnectPort.at(3)); else config->writeEntry("3Port","");
	if (lastConnectHost.at(4)!=NULL) config->writeEntry("4Host",lastConnectHost.at(4)); else config->writeEntry("4Host","");
	if (lastConnectPort.at(4)!=NULL) config->writeEntry("4Port",lastConnectPort.at(4)); else config->writeEntry("4Port","");

  config->setGroup("ANSI COLORS");
  QColor dark = view->getOutputANSIBlack();
  config->writeEntry("ANSI_Black", dark);
  dark = view->getOutputANSIDarkRed();
  config->writeEntry("ANSI_DarkRed", dark);
  dark = view->getOutputANSIDarkGreen();
  config->writeEntry("ANSI_DarkGreen", dark);
  dark = view->getOutputANSIDarkYellow();
  config->writeEntry("ANSI_DarkYellow", dark);
  dark = view->getOutputANSIDarkBlue();
  config->writeEntry("ANSI_DarkBlue", dark);
  dark = view->getOutputANSIDarkMagenta();
  config->writeEntry("ANSI_DarkMagenta", dark);
  dark = view->getOutputANSIDarkCyan();
  config->writeEntry("ANSI_DarkCyan", dark);
  dark = view->getOutputANSIDarkWhite();
  config->writeEntry("ANSI_DarkWhite", dark);

  QColor light = view->getOutputANSIGray();
  config->writeEntry("ANSI_Gray", light);
  light = view->getOutputANSILightRed();
  config->writeEntry("ANSI_LightRed", light);
  light = view->getOutputANSILightGreen();
  config->writeEntry("ANSI_LightGreen", light);
  light = view->getOutputANSILightYellow();
  config->writeEntry("ANSI_LightYellow", light);
  light = view->getOutputANSILightBlue();
  config->writeEntry("ANSI_LightBlue", light);
  light = view->getOutputANSILightMagenta();
  config->writeEntry("ANSI_LightMagenta", light);
  light = view->getOutputANSILightCyan();
  config->writeEntry("ANSI_LightCyan", light);
  light = view->getOutputANSILightWhite();
  config->writeEntry("ANSI_LightWhite", light);

  QColor bg = view->getOutputBgBlack();
  config->writeEntry("BG_Black", bg);
  bg = view->getOutputBgRed();
  config->writeEntry("BG_Red", bg);
  bg = view->getOutputBgGreen();
  config->writeEntry("BG_Green", bg);
  bg = view->getOutputBgYellow();
  config->writeEntry("BG_Yellow", bg);
  bg = view->getOutputBgBlue();
  config->writeEntry("BG_Blue", bg);
  bg = view->getOutputBgMagenta();
  config->writeEntry("BG_Magenta", bg);
  bg = view->getOutputBgCyan();
  config->writeEntry("BG_Cyan", bg);
  bg = view->getOutputBgWhite();
  config->writeEntry("BG_White", bg);
}

void KmudApp::readOptions()
{
  ///////////////////////////////////////////////////////////////////
  // read the config file entries
  KConfig *config = kapp->getConfig();

 
  config->setGroup("APPEARANCE");
  mainWindowWidth = config->readNumEntry("WindowWidth",600);
  mainWindowHeight = config->readNumEntry("WindowHeight",400);
  bViewToolbar = config->readBoolEntry("ShowToolbar", true);
  bViewStatusbar = config->readBoolEntry("ShowStatusbar", true);
  bViewAltInput = config->readBoolEntry("ShowAltInput", false);
  menu_bar_pos = (KMenuBar::menuPosition)config->readNumEntry("MenuBarPos", KMenuBar::Top); 
  tool_bar_pos = (KToolBar::BarPosition)config->readNumEntry("ToolBar_Pos", KToolBar::Top);


  config->setGroup("OPTIONS");
  mudForegroundColor=QColor(0,255,0);
  mudForegroundColor=config->readColorEntry("MudTextColor", &mudForegroundColor);
  mudBackgroundColor=QColor(0,0,0);
  mudBackgroundColor=config->readColorEntry("MudBackgroundColor", &mudBackgroundColor);
  mudFont=QFont("fixed");
  mudFont=config->readFontEntry("MudFont", &mudFont);
  inputFont=QFont("helvetica");
  inputFont=config->readFontEntry("InputFont", &inputFont);
  mudLocalEchoColor=QColor(255,0,0);
  mudLocalEchoColor=config->readColorEntry("MudLocalEchoColor", &mudLocalEchoColor);
  mudDebugHighlightColor=QColor(0, 255, 0);
  mudDebugHighlightColor=config->readColorEntry("MudDebugHighlightColor", &mudDebugHighlightColor);

  maxOutputLines = config->readNumEntry("MaxOutputLines",5000);
  maxInputLines = config->readNumEntry("MaxInputLines",1000);
  localEchoMode = config->readNumEntry("Local_Echo",2);
  inputEchoMode = config->readNumEntry("Input_Echo",2);
  separator = config->readEntry("CharSeparator", ";");
  keepLastInput = config->readBoolEntry("Keep_Last_Input",false);
  autoReconnectOption = config->readBoolEntry("AutoReconnect",true);
  useBeep = config->readBoolEntry("UseBeep",true);

  config->setGroup("HISTORY");
  lastConnectHost.clear();
  lastConnectPort.clear();
	lastConnectHost.append(config->readEntry("0Host",""));
	lastConnectPort.append(config->readEntry("0Port",""));
	lastConnectHost.append(config->readEntry("1Host",""));
	lastConnectPort.append(config->readEntry("1Port",""));
	lastConnectHost.append(config->readEntry("2Host",""));
	lastConnectPort.append(config->readEntry("2Port",""));
	lastConnectHost.append(config->readEntry("3Host",""));
	lastConnectPort.append(config->readEntry("3Port",""));
	lastConnectHost.append(config->readEntry("4Host",""));
	lastConnectPort.append(config->readEntry("4Port",""));

    config->setGroup("ANSI COLORS");
    mudANSIBlack=QColor(0, 0, 0);
    mudANSIDarkRed=QColor(128, 0, 0);
    mudANSIDarkGreen=QColor(0, 128, 0);
    mudANSIDarkYellow=QColor(128, 128, 0);
    mudANSIDarkBlue=QColor(0, 0, 128);
    mudANSIDarkMagenta=QColor(128, 0, 128);
    mudANSIDarkCyan=QColor(0, 128, 128);
    mudANSIDarkWhite=QColor(192, 192, 192);
    mudANSIGray=QColor(128, 128, 128);
    mudANSILightRed=QColor(255, 0, 0);
    mudANSILightGreen=QColor(0, 255, 0);
    mudANSILightYellow=QColor(255, 255, 0);
    mudANSILightBlue=QColor(0, 0, 255);
    mudANSILightMagenta=QColor(255, 0, 255);
    mudANSILightCyan=QColor(0, 255, 255);
    mudANSILightWhite=QColor(255, 255, 255);

    mudBgBlack=QColor(0, 0, 0);
    mudBgRed=QColor(128, 0, 0);
    mudBgGreen=QColor(0, 128, 0);
    mudBgYellow=QColor(128, 128, 0);
    mudBgBlue=QColor(0, 0, 128);
    mudBgMagenta=QColor(128, 0, 128);
    mudBgCyan=QColor(0, 128, 128);
    mudBgWhite=QColor(192, 192, 192);

    mudANSIBlack=config->readColorEntry("ANSI_Black", &mudANSIBlack);
    mudANSIDarkRed=config->readColorEntry("ANSI_DarkRed", &mudANSIDarkRed);
    mudANSIDarkGreen=config->readColorEntry("ANSI_DarkGreen", &mudANSIDarkGreen);
    mudANSIDarkYellow=config->readColorEntry("ANSI_DarkYellow", &mudANSIDarkYellow);
    mudANSIDarkBlue=config->readColorEntry("ANSI_DarkBlue", &mudANSIDarkBlue);
    mudANSIDarkMagenta=config->readColorEntry("ANSI_DarkMagenta", &mudANSIDarkMagenta);
    mudANSIDarkCyan=config->readColorEntry("ANSI_DarkCyan", &mudANSIDarkCyan);
    mudANSIDarkWhite=config->readColorEntry("ANSI_DarkWhite", &mudANSIDarkWhite);

    mudANSIGray=config->readColorEntry("ANSI_Gray", &mudANSIGray);
    mudANSILightRed=config->readColorEntry("ANSI_LightRed", &mudANSILightRed);
    mudANSILightGreen=config->readColorEntry("ANSI_LightGreen", &mudANSILightGreen);
    mudANSILightYellow=config->readColorEntry("ANSI_LightYellow", &mudANSILightYellow);
    mudANSILightBlue=config->readColorEntry("ANSI_LightBlue", &mudANSILightBlue);
    mudANSILightMagenta=config->readColorEntry("ANSI_LightMagenta", &mudANSILightMagenta);
    mudANSILightCyan=config->readColorEntry("ANSI_LightCyan", &mudANSILightCyan);
    mudANSILightWhite=config->readColorEntry("ANSI_LightWhite", &mudANSILightWhite);

    mudBgBlack=config->readColorEntry("BG_Black", &mudBgBlack);
    mudBgRed=config->readColorEntry("BG_Red", &mudBgRed);
    mudBgGreen=config->readColorEntry("BG_Green", &mudBgGreen);
    mudBgYellow=config->readColorEntry("BG_Yellow", &mudBgYellow);
    mudBgBlue=config->readColorEntry("BG_Blue", &mudBgBlue);
    mudBgMagenta=config->readColorEntry("BG_Magenta", &mudBgMagenta);
    mudBgCyan=config->readColorEntry("BG_Cyan", &mudBgCyan);
    mudBgWhite=config->readColorEntry("BG_White", &mudBgWhite);
}

/** Connect to a mud that as been setup in with the wizzard
  * eg. ID -> "Sunbright@MidnightSun"
  */
int KmudApp::connectWizzardId(QString ID)
{
	QString character;
	QString mud;
	CharDBEntry* entry;
	CMudProfile* mudPro;
	QString port;
	int pos;

	// Find the mud Title and Charcter from given ID
	pos = ID.find('@',0,false);
	character = ID.left(pos);
	mud = ID.right(ID.length()-pos-1);

	entry=doc->getEntry(character,mud);	
	if (entry!=NULL)
	{
			mudPro = doc->getMudProfile(entry->mudfile);
			
			if (mudPro!=NULL)
			{
				// Attempt to connect to the mud
				port.sprintf("%d",mudPro->getPort());		
				connectToMud(mudPro->getTitle(),entry->login,mudPro->getHostName(),port);
			}
			else
			{
				// Unable to find the entry so tell the user
				QMessageBox NotFound("KMud",ID + i18n(" can't be found"),QMessageBox::Information,
				  									 QMessageBox::Ok | QMessageBox::Default,0,0,0,"NoConnectMessageBox1",TRUE,0);
				NotFound.exec();			
			}

	}
	else
	{
			// Unable to find the entry so tell the user
			QMessageBox NotFound("KMud",ID + i18n(" can't be found"),QMessageBox::Information,
													 QMessageBox::Ok | QMessageBox::Default,0,0,0,"NoConnectMessageBox1",TRUE,0);
			NotFound.exec();			
	}
}


void KmudApp::connectToMud(QString mudTitle, QString login,QString mudHost,QString mudPort)
{
  int connectReturn=0;
  unsigned int port;

   if (mudPort=="") port=23;
    else port=mudPort.toUInt();

   connectReturn = mud->connect(mudHost,port);
   switch ( connectReturn )
   {
    case -1 : {
               QMessageBox noconnect1("kmud",i18n("Unknown Host"),QMessageBox::Information,
                                 QMessageBox::Ok | QMessageBox::Default,0,0,0,"NoConnectMessageBox1",TRUE,0);
               noconnect1.exec();
               break;
              }
    case -2 : {
               QMessageBox noconnect2("kmud",i18n("Cant create socket"),QMessageBox::Information,
                                 QMessageBox::Ok | QMessageBox::Default,0,0,0,"NoConnectMessageBox2",TRUE,0);
               noconnect2.exec();
               break;
              }
    case -3 : {
               QMessageBox noconnect3("kmud",i18n("Connect to ") + mud->getHostIP() + i18n(" timed out"),QMessageBox::Information,
                                 QMessageBox::Ok | QMessageBox::Default,0,0,0,"NoConnectMessageBox3",TRUE,0);
               noconnect3.exec();
              }
   }
   if (connectReturn>=0)
   {

    QObject::connect( mud, SIGNAL(filledBuffer(QString)), view, SLOT(slotFlushTelnetBuffer(QString)) );
    QObject::connect( view, SIGNAL(enteredInput(QString, int)), mud, SLOT(sendData(QString, int)) );
    QObject::connect( mud, SIGNAL(changedEcho(bool)), view, SLOT(setLocalEcho(bool)) );
    QObject::connect( mud, SIGNAL(changedEcho(bool)), view, SLOT(setInputEcho(bool)) );
    QObject::connect( mud, SIGNAL(closedConnection()), view, SLOT(slotMudClosed()) );
    QObject::connect( mud, SIGNAL(closedConnection()), this, SLOT(slotMudClose()) );

    toolBar()->removeItem(ID_MUD_CONNECTION_WIZARD);
    toolBar()->insertButton(Icon("mini/kmud_disconnect.xpm"), ID_MUD_CLOSE, true, i18n("Close"),toolBarConnectButtonPosition);

    if ((login!="") && (mudTitle!="")) {
        doc->setCurrentCharacterID(login+"@"+mudTitle);
        doc->setCurrentMudID(mudTitle);
        profile_menu->setItemEnabled(aliasID, true);
        profile_menu->setItemEnabled(triggerID, true);
        profile_menu->setItemEnabled(saveID, true);
        option_menu->setItemEnabled(mudOptionsID, true);
        toolBar()->setItemEnabled(aliasID, true);
        toolBar()->setItemEnabled(triggerID, true);
    }
    else
    {
        doc->setCurrentCharacterID("");
        doc->setCurrentMudID("");
        profile_menu->setItemEnabled(aliasID, false);
        profile_menu->setItemEnabled(triggerID, false);
        profile_menu->setItemEnabled(saveID, false);
        option_menu->setItemEnabled(mudOptionsID, false);
        toolBar()->setItemEnabled(aliasID, false);
        toolBar()->setItemEnabled(triggerID, false);
    }

    if ( (currentLogin!="") && (currentMudTitle!="") )
     setCaption(currentLogin+"@"+currentMudTitle+" - Kmud " VERSION);
		 else setCaption(mudHost+" - Kmud " VERSION);

    if ( (lastConnectHost.last()!=mudHost) || (lastConnectPort.last()!=mudPort) )
    {
     lastConnectHost.removeFirst();
     lastConnectPort.removeFirst();
     lastConnectHost.append(mudHost);
     lastConnectPort.append(mudPort);
    }
   }

	// Update the connection pixmap on the status bar
	connectIcon->setPixmap(Icon("mini/mini-connected.xpm"));
	// Setup and start the connection timer
	statusBar()->changeItem(i18n("00:00:00"), ID_STATUS_TIME );
	connectTimer->start(1000,FALSE);

	// Make a copy of the prameters to this function so that the autoreconnect
	// is able to reconncect.
	lastTitle = mudTitle;
	lastLogin = login;
	lastHost = mudHost;
	lastPort = mudPort;

	autoReconnect = true;	

}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////

void KmudApp::slotMudNewWindow()
{
  slotStatusMsg(i18n("Opening a new Application window..."));
  (new KmudApp)->show();
  slotStatusMsg(i18n("Ready."));
}

void KmudApp::slotMudConnectionWizard()
{
  slotStatusMsg(i18n("Connection Wizard..."));

	DlgConnectionWizard d(doc,this,"ConnectionWizardDialog");

  switch (d.exec())
  {
   case 1: if ( !mud->isConnected() )
           {
            /* Read in MUD options */
            doc->getMudProfile(doc->current()->mudfile)->readData();
            interpretType = doc->getMudProfile(doc->current()->mudfile)->getInterpretationType();
            view->setInterpretType(interpretType);
            /* Finished reading in options*/

            connectToMud(d.getMudTitle(),d.getLogin(),d.getMudHost(),d.getMudPort());
            if ((d.getLogin()!="") && (d.getMudTitle()!="")) {
                currentLogin=d.getLogin();
                currentMudTitle=d.getMudTitle();
                doc->setCurrentMudID(d.getMudID());
                doc->setCurrentCharacterID(d.getCharacterID());
                profile_menu->setItemEnabled(aliasID, true);
                profile_menu->setItemEnabled(triggerID, true);
                profile_menu->setItemEnabled(saveID, true);
                option_menu->setItemEnabled(mudOptionsID, true);
                toolBar()->setItemEnabled(aliasID, true);
                toolBar()->setItemEnabled(triggerID, true);
            }
            else
            {
                currentLogin="";
                currentMudTitle="";
                doc->setCurrentCharacterID("");
                doc->setCurrentMudID("");
                profile_menu->setItemEnabled(aliasID, false);
                profile_menu->setItemEnabled(triggerID, false);
                profile_menu->setItemEnabled(saveID, false);
                option_menu->setItemEnabled(mudOptionsID, false);
                toolBar()->setItemEnabled(aliasID, false);
                toolBar()->setItemEnabled(triggerID, false);
            }
           } // end if mud->isConnected()
           break;
   case 2: if ( !mud->isConnected() )
           {
            /* Read in MUD options */
            doc->getMudProfile(doc->current()->mudfile)->readData();
            interpretType = doc->getMudProfile(doc->current()->mudfile)->getInterpretationType();
            view->setInterpretType(interpretType);
            /* Finished reading in options*/

            if ((d.getLogin()!="") && (d.getMudTitle()!="")) {
                currentLogin=d.getLogin();
                currentMudTitle=d.getMudTitle();
                doc->setCurrentMudID(d.getMudID());
                doc->setCurrentCharacterID(d.getCharacterID());
                profile_menu->setItemEnabled(aliasID, true);
                profile_menu->setItemEnabled(triggerID, true);
                profile_menu->setItemEnabled(saveID, true);
                option_menu->setItemEnabled(mudOptionsID, true);
                toolBar()->setItemEnabled(aliasID, true);
                toolBar()->setItemEnabled(triggerID, true);
            } else {
                currentLogin="";
                currentMudTitle="";
                doc->setCurrentCharacterID("");
                doc->setCurrentMudID("");
                profile_menu->setItemEnabled(aliasID, false);
                profile_menu->setItemEnabled(triggerID, false);
                profile_menu->setItemEnabled(saveID, false);
                option_menu->setItemEnabled(mudOptionsID, false);
                toolBar()->setItemEnabled(aliasID, false);
                toolBar()->setItemEnabled(triggerID, false);
            }

            if ( (currentLogin!="") && (currentMudTitle!="") )
             setCaption(currentLogin+"@"+currentMudTitle+" - Kmud " VERSION);
           } // end if mud->isConnected()
           break;
  }


  slotStatusMsg(i18n("Ready."));
}


void KmudApp::slotMudConnect()
{
  slotStatusMsg(i18n("Connecting to MUD..."));

  DlgConnect d(this,"ConnectDialog");

	d.setHost(lastConnectHost.last());
	d.setPort(lastConnectPort.last());

  if ( !mud->isConnected() && d.exec())
  {
   connectToMud("","",d.getHost(),d.getPort());
   interpretType = 0; // auto detect by default
  } // end if mud->isConnected()


  slotStatusMsg(i18n("Ready."));
}

void KmudApp::slotMudClose()
{
  slotStatusMsg(i18n("Closing connection..."));
	
  QObject::disconnect( mud, SIGNAL(filledBuffer(QString)), view, SLOT(slotFlushTelnetBuffer(QString)) );
  QObject::disconnect( view, SIGNAL(enteredInput(QString, int)), mud, SLOT(sendData(QString, int)) );
  QObject::disconnect( mud, SIGNAL(changedEcho(bool)), view, SLOT(setLocalEcho(bool)) );
  QObject::disconnect( mud, SIGNAL(changedEcho(bool)), view, SLOT(setInputEcho(bool)) );
  QObject::disconnect( mud, SIGNAL(closedConnection()), this, SLOT(slotMudClose()) );
  QObject::disconnect( mud, SIGNAL(closedConnection()), view, SLOT(slotMudClosed()) );
  view->setLocalEcho(true);
  view->setInputEcho(true);

  if (mud->isConnected())
   {
    mud->close();
   }

  if (toolBar()->getButton(ID_MUD_CLOSE)!=NULL)
  {
   toolBar()->removeItem(ID_MUD_CLOSE);
   toolBar()->insertButton(Icon("mini/kmud_connectionwizard.xpm"), ID_MUD_CONNECTION_WIZARD, true, i18n("Connect"),toolBarConnectButtonPosition);
  }

  doc->setCurrentCharacterID("");
  doc->setCurrentMudID("");
  profile_menu->setItemEnabled(aliasID, false);
  profile_menu->setItemEnabled(triggerID, false);
  profile_menu->setItemEnabled(saveID, false);
  option_menu->setItemEnabled(mudOptionsID, false);
  toolBar()->setItemEnabled(aliasID, false);
  toolBar()->setItemEnabled(triggerID, false);

	setCaption("Kmud " VERSION);

	slotStatusMsg(i18n("Ready."));
	connectIcon->setPixmap(Icon("mini/mini-connect.xpm"));
  statusBar()->changeItem(i18n("00:00:00"), ID_STATUS_TIME );
	connectTimer->stop();

	if (autoReconnect && autoReconnectOption)
	{
	  DlgReconnect d(this,"ReconnectDialog");

	  if (d.exec())
		{
			connectToMud(lastTitle,lastLogin,lastHost,lastPort);	
		}
	}
}


void KmudApp::slotMudCloseWindow()
{
  close();
}
void KmudApp::slotMudQuit()
{ 

  ///////////////////////////////////////////////////////////////////
  // exits the Application
  if(this->queryExit())
    {
      saveOptions();
      KTMainWindow::deleteAll();
      kapp->quit();
    }
  else
    slotStatusMsg(i18n("Ready."));
    return;
}


void KmudApp::slotViewToolBar()
{
  ///////////////////////////////////////////////////////////////////
  // turn Toolbar on or off
  bViewToolbar=!bViewToolbar;
  menuBar()->setItemChecked(ID_VIEW_TOOLBAR, bViewToolbar);
  enableToolBar(KToolBar::Toggle,0);
  slotStatusMsg(i18n("Ready."));

}

void KmudApp::slotViewStatusBar()
{
  ///////////////////////////////////////////////////////////////////
  //turn Statusbar on or off
  bViewStatusbar=!bViewStatusbar;
  menuBar()->setItemChecked(ID_VIEW_STATUSBAR, bViewStatusbar);
  statusBar()->clear();
  enableStatusBar();
  slotStatusMsg(i18n("Ready."));
}

void KmudApp::slotViewAltInput()
{
  ///////////////////////////////////////////////////////////////////
  //turn alternative Iinput on or off
  bViewAltInput=!bViewAltInput;
  menuBar()->setItemChecked(ID_VIEW_ALTINPUT, bViewAltInput);

  view->setAltInput(bViewAltInput);

  slotStatusMsg(i18n("Ready."));
}

void KmudApp::slotViewMapper()
{
	////////////////////////////////////////////////////////////////////
	//show the auto mapper
	mapper = new CMapWindow();

	slotStatusMsg(i18n("Ready."));	
}

void KmudApp::slotOptionFont()
{
  slotStatusMsg(i18n("Changing font settings..."));

  DlgOptionFonts d(this,"FontSetupDialog");

  readOptions();

  d.setOutputFont(view->getOutputFont());
  d.setInputFont(view->getInputFont());

  if (d.exec())
  {
   view->setOutputFont(d.getOutputFont());
	 mudFont=d.getOutputFont();
   view->setInputFont(d.getInputFont());
	 inputFont=d.getInputFont();

   saveOptions();
  }

  slotStatusMsg(i18n("Ready."));
}

void KmudApp::slotOptionColors()
{
  slotStatusMsg(i18n("Changing color settings..."));

  DlgOptionColors d(this,"ColorSetupDialog", interpretType);


  readOptions();
  d.setForegroundColor(view->getOutputForegroundColor());
  d.setBackgroundColor(view->getOutputBackgroundColor());
  d.setLocalEchoColor(view->getLocalEchoColor());

  d.setDebugHighlightColor(view->getDebugHighlightColor());

  d.setANSIBlack(view->getOutputANSIBlack());
  d.setANSIDarkRed(view->getOutputANSIDarkRed());
  d.setANSIDarkGreen(view->getOutputANSIDarkGreen());
  d.setANSIDarkYellow(view->getOutputANSIDarkYellow());
  d.setANSIDarkBlue(view->getOutputANSIDarkBlue());
  d.setANSIDarkMagenta(view->getOutputANSIDarkMagenta());
  d.setANSIDarkCyan(view->getOutputANSIDarkCyan());
  d.setANSIDarkWhite(view->getOutputANSIDarkWhite());

  d.setANSIGray(view->getOutputANSIGray());
  d.setANSILightRed(view->getOutputANSILightRed());
  d.setANSILightGreen(view->getOutputANSILightGreen());
  d.setANSILightYellow(view->getOutputANSILightYellow());
  d.setANSILightBlue(view->getOutputANSILightBlue());
  d.setANSILightMagenta(view->getOutputANSILightMagenta());
  d.setANSILightCyan(view->getOutputANSILightCyan());
  d.setANSILightWhite(view->getOutputANSILightWhite());

  d.setBgBlack(view->getOutputBgBlack());
  d.setBgRed(view->getOutputBgRed());
  d.setBgGreen(view->getOutputBgGreen());
  d.setBgYellow(view->getOutputBgYellow());
  d.setBgBlue(view->getOutputBgBlue());
  d.setBgMagenta(view->getOutputBgMagenta());
  d.setBgCyan(view->getOutputBgCyan());
  d.setBgWhite(view->getOutputBgWhite());

  if (d.exec())
   {
    mudForegroundColor = d.getForegroundColor();
    mudBackgroundColor = d.getBackgroundColor();
    mudLocalEchoColor = d.getLocalEchoColor();
    mudDebugHighlightColor = d.getDebugHighlightColor();
    view->setOutputForegroundColor(mudForegroundColor);
    view->setOutputBackgroundColor(mudBackgroundColor);
    view->setInputBackgroundColor(mudBackgroundColor);
    view->setLocalEchoColor(mudLocalEchoColor);
    view->setDebugHighlightColor(mudDebugHighlightColor);


    mudANSIBlack = d.getANSIBlack();
    mudANSIDarkRed = d.getANSIDarkRed();
    mudANSIDarkGreen = d.getANSIDarkGreen();
    mudANSIDarkYellow = d.getANSIDarkYellow();
    mudANSIDarkBlue = d.getANSIDarkBlue();
    mudANSIDarkMagenta = d.getANSIDarkMagenta();
    mudANSIDarkCyan = d.getANSIDarkCyan();
    mudANSIDarkWhite = d.getANSIDarkWhite();

    mudANSIGray = d.getANSIGray();
    mudANSILightRed = d.getANSILightRed();
    mudANSILightGreen = d.getANSILightGreen();
    mudANSILightYellow = d.getANSILightYellow();
    mudANSILightBlue = d.getANSILightBlue();
    mudANSILightMagenta = d.getANSILightMagenta();
    mudANSILightCyan = d.getANSILightCyan();
    mudANSILightWhite = d.getANSILightWhite();

    mudBgBlack = d.getBgBlack();
    mudBgRed = d.getBgRed();
    mudBgGreen = d.getBgGreen();
    mudBgYellow = d.getBgYellow();
    mudBgBlue = d.getBgBlue();
    mudBgMagenta = d.getBgMagenta();
    mudBgCyan = d.getBgCyan();
    mudBgWhite = d.getBgWhite();

    view->setOutputANSIBlack(mudANSIBlack);
    view->setOutputANSIDarkRed(mudANSIDarkRed);
    view->setOutputANSIDarkGreen(mudANSIDarkGreen);
    view->setOutputANSIDarkYellow(mudANSIDarkYellow);
    view->setOutputANSIDarkBlue(mudANSIDarkBlue);
    view->setOutputANSIDarkMagenta(mudANSIDarkMagenta);
    view->setOutputANSIDarkCyan(mudANSIDarkCyan);
    view->setOutputANSIDarkWhite(mudANSIDarkWhite);

    view->setOutputANSIGray(mudANSIGray);
    view->setOutputANSILightRed(mudANSILightRed);
    view->setOutputANSILightGreen(mudANSILightGreen);
    view->setOutputANSILightYellow(mudANSILightYellow);
    view->setOutputANSILightBlue(mudANSILightBlue);
    view->setOutputANSILightMagenta(mudANSILightMagenta);
    view->setOutputANSILightCyan(mudANSILightCyan);
    view->setOutputANSILightWhite(mudANSILightWhite);

    view->setOutputBgBlack(mudBgBlack);
    view->setOutputBgRed(mudBgRed);
    view->setOutputBgGreen(mudBgGreen);
    view->setOutputBgYellow(mudBgYellow);
    view->setOutputBgBlue(mudBgBlue);
    view->setOutputBgMagenta(mudBgMagenta);
    view->setOutputBgCyan(mudBgCyan);
    view->setOutputBgWhite(mudBgWhite);

    saveOptions();
   }

  slotStatusMsg(i18n("Ready."));
}

void KmudApp::slotOptionMud()
{
    if (doc->getCurrentMudID() == "") {
        QMessageBox::information(this, "Error", "There is no active MUD.  Please use\nthe Connection Wizard to select one.");
        return;
    }

    slotStatusMsg(i18n("Changing MUD Settings"));
    DlgOptionMud d(doc, this, "Mud Settings");

//    d.setInterpretType(interpretType);
    if (d.exec())
    {
        interpretType = d.getInterpretType();
        view->setInterpretType(interpretType);
    }

    slotStatusMsg(i18n("Ready."));
}

void KmudApp::slotOptionPreferences()
{
  slotStatusMsg(i18n("Changing preferences..."));

	DlgOptionPreferences d(this, "Preferences");

    d.setAutoReconnect(autoReconnectOption);
    d.setKeepLastInput(keepLastInput);
    d.setLocalEchoMode(localEchoMode);
    d.setInputEchoMode(inputEchoMode);
    d.setOutputSize(maxOutputLines);
    d.setInputSize(maxInputLines);
    d.setSeparator(separator);
    d.setUseBeep(useBeep);

  if (d.exec())
  {
   localEchoMode=d.getLocalEchoMode();
   inputEchoMode=d.getInputEchoMode();
   maxOutputLines=d.getOutputSize();
   maxInputLines=d.getInputSize();
   keepLastInput=d.getKeepLastInput();
	 autoReconnectOption=d.getAutoReconnect();
	 separator=d.getSeparator();
	 useBeep=d.getUseBeep();

   view->setLocalEchoMode(localEchoMode);
   view->setInputEchoMode(inputEchoMode);
   view->setMaxHistoryLines(maxInputLines);
   view->setMaxMudViewLines(maxOutputLines);
   view->setKeepLastInput(keepLastInput);
   view->setSeparator(separator);
   view->setUseBeep(useBeep);

   saveOptions();
  }

  slotStatusMsg(i18n("Ready."));
}

void KmudApp::slotProfileAliases()
{
  if (doc->getCurrentCharacterID() == "") {
    QMessageBox::information(this, "Error", "There is no active profile.\nPlease connect through Connection Wizard.");
    return;
  }

  slotStatusMsg(i18n("Editing aliases..."));

  DlgProfileAliases d(doc, this, "Aliases");


  if (d.exec())
  {
  }

  slotStatusMsg(i18n("Ready."));
}

void KmudApp::slotProfileTriggers()
{
  if (doc->getCurrentCharacterID() == "") {
    QMessageBox::information(this, "Error", "There is no active profile.\nPlease connect through Connection Wizard.");
    return;
  }

  slotStatusMsg(i18n("Editing triggers..."));

  DlgProfileTriggers d(doc, this, "Triggers");

  if (d.exec())
  {
  }

  slotStatusMsg(i18n("Ready."));
}

void KmudApp::slotProfileSave()
{
  slotStatusMsg(i18n("Saving profile..."));


  slotStatusMsg(i18n("Ready."));
}

void KmudApp::slotStatusMsg(const char *text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message permanently
  statusBar()->clear();
  statusBar()->changeItem(text, ID_STATUS_MSG );
}


void KmudApp::slotStatusHelpMsg(const char *text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message of whole statusbar temporary (text, msec)
  statusBar()->message(text, 2000);
}

	
void KmudApp::slotConnectionTime(void)
{	
	///////////////////////////////////////////////////////////////////
	// Update the staus bar to show the new amount of time a user
  // has been connected to the mud
	char time[10];	

	connectSec++;
	if (connectSec==60)
	{
		connectMin++;
		connectSec=0;
	}
	if (connectMin==60)
	{
		connectHour++;
		connectMin=0;
	}
	
	// construct the connection status string a
  // and add leading zeros to the hour,mins and seconds.	
	if (connectHour<10)
		sprintf(time,"0%d:",connectHour);
	else
		sprintf(time,"%d:",connectHour);
	if (connectMin<10)
		sprintf(time,"%s0%d:",time,connectMin);
	else
		sprintf(time,"%s%d:",time,connectMin);
	if (connectSec<10)
		sprintf(time,"%s0%d",time,connectSec);
	else
		sprintf(time,"%s%d",time,connectSec);

	statusBar()->changeItem(i18n(time), ID_STATUS_TIME );
}

void KmudApp::commandCallback(int id_){
  switch (id_){
    ON_CMD(ID_MUD_NEW_WINDOW,          slotMudNewWindow())
    ON_CMD(ID_MUD_CONNECTION_WIZARD,   slotMudConnectionWizard())
    ON_CMD(ID_MUD_CONNECT,             slotMudConnect())
    ON_CMD(ID_MUD_CLOSE,               autoReconnect = false;slotMudClose())
    ON_CMD(ID_MUD_CLOSE_WINDOW,        slotMudCloseWindow())
    ON_CMD(ID_MUD_QUIT,                slotMudQuit())

    ON_CMD(ID_VIEW_TOOLBAR,             slotViewToolBar())
    ON_CMD(ID_VIEW_STATUSBAR,           slotViewStatusBar())
    ON_CMD(ID_VIEW_ALTINPUT,            slotViewAltInput())
		ON_CMD(ID_VIEW_MAPPER,              slotViewMapper())

    ON_CMD(ID_OPTION_FONT,              slotOptionFont())
    ON_CMD(ID_OPTION_COLORS,            slotOptionColors())
    ON_CMD(ID_OPTION_PREFERENCES,       slotOptionPreferences())
    ON_CMD(ID_OPTION_MUD,               slotOptionMud())
    ON_CMD(ID_OPTION_SAVE,              saveOptions())

    ON_CMD(ID_PROFILE_ALIASES,           slotProfileAliases())
    ON_CMD(ID_PROFILE_TRIGGERS,          slotProfileTriggers())
    ON_CMD(ID_PROFILE_SAVE,              slotProfileSave())
  }
}

void KmudApp::statusCallback(int id_){
  switch (id_){
    ON_STATUS_MSG(ID_MUD_NEW_WINDOW,   i18n("Opens a new application window"))
    ON_STATUS_MSG(ID_MUD_CONNECTION_WIZARD,      i18n("Invokes the Connection Wizard"))
    ON_STATUS_MSG(ID_MUD_CONNECT,      i18n("Connect to a MUD"))
    ON_STATUS_MSG(ID_MUD_CLOSE,        i18n("Closes a connection"))
    ON_STATUS_MSG(ID_MUD_CLOSE_WINDOW, i18n("Closes the current window"))
    ON_STATUS_MSG(ID_MUD_QUIT,         i18n("Exits the program"))

    ON_STATUS_MSG(ID_VIEW_TOOLBAR,      i18n("Enables / disables the actual Toolbar"))
    ON_STATUS_MSG(ID_VIEW_STATUSBAR,    i18n("Enables / disables the Statusbar"))
    ON_STATUS_MSG(ID_VIEW_ALTINPUT,     i18n("Enables / disables the alternative Input"))
		ON_STATUS_MSG(ID_VIEW_MAPPER,       i18n("Enables / disables the auto mapper"))

    ON_STATUS_MSG(ID_OPTION_FONT,       i18n("Change font settings"))
    ON_STATUS_MSG(ID_OPTION_COLORS,     i18n("Change color settings"))
    ON_STATUS_MSG(ID_OPTION_PREFERENCES,     i18n("Change preferences"))
    ON_STATUS_MSG(ID_OPTION_MUD,        i18n("Change MUD Settings"))
    ON_STATUS_MSG(ID_OPTION_SAVE,       i18n("Save options"))

    ON_STATUS_MSG(ID_PROFILE_ALIASES,     i18n("Edit aliases profile"))
    ON_STATUS_MSG(ID_PROFILE_TRIGGERS,    i18n("Edit triggers profile"))
    ON_STATUS_MSG(ID_PROFILE_SAVE,       i18n("Save profile"))
  }
}



void KmudApp::closeEvent(QCloseEvent* ce)
{
 saveOptions(); // save options before we quit
 KTMainWindow::closeEvent(ce);
}




