/***************************************************************************
                          kmud.h
                    -------------------
    description          : Main Application Class
    begin                : Sun Jun 20 14:17:54 MEST 1999
    copyright            : (C) 1999 by Stephan Uhlmann
    email                : suhlmann@gmx.de                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#ifndef KMUD_H
#define KMUD_H 
 

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

// include files for QT
#include <qdialog.h>
#include <qmessagebox.h>
#include <qpushbutton.h>
#include <qprinter.h>
#include <qpainter.h>
#include <qmultilinedit.h>
#include <qlineedit.h>
#include <qstrlist.h>
#include <qtimer.h>

// include files for KDE 
#include <kapp.h> 
#include <ktmainwindow.h> 
#include <kaccel.h>
#include <kiconloader.h>

// application specific includes
#include "kmudview.h"
#include "kmuddoc.h"
#include "resource.h"

#include "ctelnet.h"
#include "dlgconnect.h"
#include "dlgoptionfonts.h"
#include "dlgoptioncolors.h"
#include "dlgoptionpreferences.h"
#include "dlgconnectionwizard.h"
#include "dlgreconnect.h"
#include "cmapwindow.h"
#include "dlgprofilealiases.h"
#include "dlgprofiletriggers.h"
#include "dlgprofilemacros.h"
#include "dlgoptionmud.h"

/**
	* This class is the base class for Kmud.
	* It sets up the main  window and reads the config file as well as
	* providing a menubar, toolbar  and statusbar. For the main view,
	* an instance of class KmudView is created which creates the view.
  *@author Stephan Uhlmann
	*/

class KmudApp : public KTMainWindow
{
  Q_OBJECT

public:
  /** construtor */
  KmudApp(); 
  /** destructor */
  ~KmudApp();
  /** initMenuBar creates the menu_bar and inserts the menuitems */
  void initMenuBar();
  /** this creates the toolbars. Change the toobar look and add new toolbars in this
    * function */
  void initToolBar();
  /** setup the statusbar */
  void initStatusBar();
  /** setup the mainview*/
  void initView();
	/** setup the connection timer */
	void initTimer();
  /** save the app-specific options on slotAppExit or by an Options dialog */	
  void saveOptions();
  /** read the app-specific options on init() or by an Options dialog */
  void readOptions();
  /** enable menuentries/toolbar items */
  void enableCommand(int id_);
  /** disable menuentries/toolbar items */
  void disableCommand(int id_);
  /** does the connect to a mud */
  void connectToMud(QString mudTitle, QString login,QString mudHost,QString mudPort);
  /** Connect to a mud that as been setup in with the wizzard
    * eg. ID -> "Sunbright@MidnightSun"
    */
  int connectWizzardId(QString ID);



public slots:
  /** switch argument for slot selection by menu or toolbar ID */
  void commandCallback(int id_);
  /** switch argument for Statusbar help entries on slot selection */
  void statusCallback(int id_);
  /** open a new application window */
  void slotMudNewWindow();
  /** popup connection wizard */
  void slotMudConnectionWizard();
  /** connect to mud */
  void slotMudConnect();
  /** close the actual connection */
  void slotMudClose();
  /** close the current window */
  void slotMudCloseWindow();
  /** exits the application */
  void slotMudQuit();


  /** toggle the toolbar*/
  void slotViewToolBar();
  /** toggle the statusbar*/
  void slotViewStatusBar();
  /** toggle the alternative input*/
  void slotViewAltInput();
	/** toggle the auto mapper */
	void slotViewMapper();


  /** setup font*/
  void slotOptionFont();
  /** setup colors*/
  void slotOptionColors();
  /** Preferences*/
  void slotOptionPreferences();
  /** MUD Settings */
  void slotOptionMud();

  /** save profile*/
  void slotProfileSave();
  /** aliases profile*/
  void slotProfileAliases();
  /** triggers profile*/
  void slotProfileTriggers();

  /** change the status message to text */
  void slotStatusMsg(const char *text);
  /** change the status message of the whole statusbar temporary */
  void slotStatusHelpMsg(const char *text);

	/** update the connection time */
	void slotConnectionTime(void);

protected:
  /** overloaded closeEvent */
	void closeEvent(QCloseEvent*);

private:
	/** If set to false then auto reconnect is disabled temporialy */
	bool autoReconnect;

	/** These varables are used to store the details of
    * that last mud that was connected to
		*/
	QString lastTitle;
	QString lastLogin;
	QString lastHost;
	QString lastPort;

  /** The connected/disconected icon for the status bar */
  QLabel *connectIcon;
  /** file_menu contains all items of the menubar entry "File" */
  QPopupMenu *mud_menu;
  /** edit_menu contains all items of the menubar entry "Edit" */
  QPopupMenu *edit_menu;
  /** view_menu contains all items of the menubar entry "View" */
  QPopupMenu *view_menu;

  /** option_menu contains all items of the menubar entry "Option" */
  QPopupMenu *option_menu;

  /** profile_menu contains all items of the menubar entry "Profile" */
  QPopupMenu *profile_menu;

  /** help_menu contains all items of the menubar entry "Help" */
  QPopupMenu *help_menu;

  /** view is the main widget which represents your working area. The View
    * class should handle all events of the view widget.  It is kept empty so
    * you can create your view according to your application's needs by
    * changing the view class. */
  KmudView *view;

	/** This is a pointer to the main auto mapper widget */
 	CMapWindow *mapper;

  /** doc represents your actual document and is created only once. It keeps
    * information such as filename and does the serialization of your files.
    */
  KmudDoc *doc;

	/** Telnet connection that this instance of KmudApp handles */
  CTelnet *mud;



  QColor mudForegroundColor;
  QColor mudBackgroundColor;
  QFont mudFont;
  QColor mudLocalEchoColor;
  QFont inputFont;

  QColor mudDebugHighlightColor;

  QColor mudANSIBlack;
  QColor mudANSIDarkRed;
  QColor mudANSIDarkGreen;
  QColor mudANSIDarkYellow;
  QColor mudANSIDarkBlue;
  QColor mudANSIDarkMagenta;
  QColor mudANSIDarkCyan;
  QColor mudANSIDarkWhite;

  QColor mudANSIGray;
  QColor mudANSILightRed;
  QColor mudANSILightGreen;
  QColor mudANSILightYellow;
  QColor mudANSILightBlue;
  QColor mudANSILightMagenta;
  QColor mudANSILightCyan;
  QColor mudANSILightWhite;

  QColor mudBgBlack;
  QColor mudBgRed;	
  QColor mudBgGreen;	
  QColor mudBgYellow;	
  QColor mudBgBlue;	
  QColor mudBgMagenta;	
  QColor mudBgCyan;	
  QColor mudBgWhite;	
	
	QStrList lastConnectHost;
	QStrList lastConnectPort;


	int mainWindowWidth;
	int mainWindowHeight;

	/** Interpretation Type */
	int interpretType;
	
  /** flag if toolbar is visible or not. Used for kconfig and checking the
    * view-menu entry view toolbar. bViewStatusbar does the same for the
    * statusbar.
    */
  bool bViewToolbar;
  bool bViewStatusbar;
  bool bViewAltInput;

	bool autoReconnectOption;
  bool keepLastInput;
	int	maxOutputLines;
	int	maxInputLines;
	int localEchoMode;
	int inputEchoMode;

  int toolBarConnectButtonPosition;


  /** used for KConfig to store and restore menubar position. Same with
    * tool_bar_pos. If more menubars or toolbars are created, you should add
    * positions as well and implement them in saveOptions() and readOptions().
    */
  KMenuBar::menuPosition menu_bar_pos;
  KToolBar::BarPosition tool_bar_pos;
  /** These variables is used to time how long a user is connected to the mud. */
  QTimer *connectTimer;
	int connectHour;
	int connectMin;
	int connectSec;

	/** The separator character */
    QString separator;

    /** Whether or not to use the \007 beep command */
    bool useBeep;

    /** ID of Aliases in the Profile menu */
    int aliasID;
    /** ID of Triggers in the Profile menu */
    int triggerID;
    /** ID of SAVE in the Profile menu */
    int saveID;
    /** ID of Mud in the Options menu */
    int mudOptionsID;

    /** ID of aliases on the ToolBar */
    int aliasToolID;
    /** ID of triggers on the ToolBar */
    int triggerToolID;


    QString currentLogin;
    QString currentMudTitle;
};
 
#endif // KMUD_H
 
 
 
 
 






