/***************************************************************************
                          kmudview.cpp
                      -------------------
    description          : View Class (Main View)
    begin                : Sun Jun 20 14:17:54 MEST 1999
    copyright            : (C) 1999 by Stephan Uhlmann
    email                : suhlmann@gmx.de                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include "kmudview.h"

KmudView::KmudView(KApplication* a,KmudDoc* doc,QWidget *parent)
	: QWidget(parent){

    document = doc;
	
  ///////////////////////////////////////////////////////////////////
  // setup my widgets

  output = new CMudView(document, this,"output");
  output->setFont(QFont("courier",12));
  output->setBackgroundMode(NoBackground);
  input = new CMudLineEdit(document, this,"input");
  input->setBackgroundMode(PaletteBackground);
  input->setFocus();
  input2 = new CMudLineEdit(document, this,"input2");
  input2->setBackgroundMode(PaletteBackground);

  setLocalEchoColor(QColor(0,0,0));

  setLocalEcho(true);
  setLocalEchoMode(2);
  setInputEcho(true);
  setInputEchoMode(2);

  QObject::connect(input, SIGNAL(returnPressed()),SLOT(slotInputReturnPressed()));
  QObject::connect(input2,SIGNAL(returnPressed()),SLOT(slotInput2ReturnPressed()));
  QObject::connect(input, SIGNAL(scrollKeyPressed(QKeyEvent *)),output,SLOT(slotScrollKey(QKeyEvent *)));
  QObject::connect(input2,SIGNAL(scrollKeyPressed(QKeyEvent *)),output,SLOT(slotScrollKey(QKeyEvent *)));
  QObject::connect(input, SIGNAL(macroPressed(QString)), SLOT(slotMacroPressed(QString)));
  QObject::connect(input2,SIGNAL(macroPressed(QString)), SLOT(slotMacroPressed(QString)));
}

KmudView::~KmudView(){
	// no need to delete widgets that are children of this class
	// since they will be destroyed when destroying the parent
	// see: ~QObject()
}


void KmudView::setOutputForegroundColor(QColor fgc)
{
 output->setForeground(fgc);
}

QColor KmudView::getOutputForegroundColor()
{
 return output->getForeground();
}

void KmudView::setOutputBackgroundColor(QColor bgc)
{
 output->setBackground(bgc);
}

QColor KmudView::getOutputBackgroundColor()
{
 return output->getBackground();
}


void KmudView::setOutputFont( QFont font )
{
 output->setFont(font);
}

QFont KmudView::getOutputFont()
{
 return output->getFont();
}

void KmudView::setInterpretType(int i)
{ output->setInterpretType(i); }
int KmudView::getInterpretType()
{ return output->getInterpretType(); }

void KmudView::setOutputANSIBlack(QColor color)
{ output->setOutputANSIBlack(color); }
QColor KmudView::getOutputANSIBlack()
{ return output->getOutputANSIBlack(); }

void KmudView::setOutputANSIDarkRed(QColor color)
{ output->setOutputANSIDarkRed(color); }
QColor KmudView::getOutputANSIDarkRed()
{ return output->getOutputANSIDarkRed(); }

void KmudView::setOutputANSIDarkGreen(QColor color)
{ output->setOutputANSIDarkGreen(color); }
QColor KmudView::getOutputANSIDarkGreen()
{ return output->getOutputANSIDarkGreen(); }

void KmudView::setOutputANSIDarkYellow(QColor color)
{ output->setOutputANSIDarkYellow(color); }
QColor KmudView::getOutputANSIDarkYellow()
{ return output->getOutputANSIDarkYellow(); }

void KmudView::setOutputANSIDarkBlue(QColor color)
{ output->setOutputANSIDarkBlue(color); }
QColor KmudView::getOutputANSIDarkBlue()
{ return output->getOutputANSIDarkBlue(); }

void KmudView::setOutputANSIDarkMagenta(QColor color)
{ output->setOutputANSIDarkMagenta(color); }
QColor KmudView::getOutputANSIDarkMagenta()
{ return output->getOutputANSIDarkMagenta(); }

void KmudView::setOutputANSIDarkCyan(QColor color)
{ output->setOutputANSIDarkCyan(color); }
QColor KmudView::getOutputANSIDarkCyan()
{ return output->getOutputANSIDarkCyan(); }

void KmudView::setOutputANSIDarkWhite(QColor color)
{ output->setOutputANSIDarkWhite(color); }
QColor KmudView::getOutputANSIDarkWhite()
{ return output->getOutputANSIDarkWhite(); }

// LIGHT COLORS

void KmudView::setOutputANSIGray(QColor color)
{ output->setOutputANSIGray(color); }
QColor KmudView::getOutputANSIGray()
{ return output->getOutputANSIGray(); }

void KmudView::setOutputANSILightRed(QColor color)
{ output->setOutputANSILightRed(color); }
QColor KmudView::getOutputANSILightRed()
{ return output->getOutputANSILightRed(); }

void KmudView::setOutputANSILightGreen(QColor color)
{ output->setOutputANSILightGreen(color); }
QColor KmudView::getOutputANSILightGreen()
{ return output->getOutputANSILightGreen(); }

void KmudView::setOutputANSILightYellow(QColor color)
{ output->setOutputANSILightYellow(color); }
QColor KmudView::getOutputANSILightYellow()
{ return output->getOutputANSILightYellow(); }

void KmudView::setOutputANSILightBlue(QColor color)
{ output->setOutputANSILightBlue(color); }
QColor KmudView::getOutputANSILightBlue()
{ return output->getOutputANSILightBlue(); }

void KmudView::setOutputANSILightMagenta(QColor color)
{ output->setOutputANSILightMagenta(color); }
QColor KmudView::getOutputANSILightMagenta()
{ return output->getOutputANSILightMagenta(); }

void KmudView::setOutputANSILightCyan(QColor color)
{ output->setOutputANSILightCyan(color); }
QColor KmudView::getOutputANSILightCyan()
{ return output->getOutputANSILightCyan(); }

void KmudView::setOutputANSILightWhite(QColor color)
{ output->setOutputANSILightWhite(color); }
QColor KmudView::getOutputANSILightWhite()
{ return output->getOutputANSILightWhite(); }


/** for background colors */
void KmudView::setOutputBgBlack(QColor color)
{ output->setOutputBgBlack(color); }
QColor KmudView::getOutputBgBlack()
{ return output->getOutputBgBlack(); }

void KmudView::setOutputBgRed(QColor color)
{ output->setOutputBgRed(color); }
QColor KmudView::getOutputBgRed()
{ return output->getOutputBgRed(); }

void KmudView::setOutputBgGreen(QColor color)
{ output->setOutputBgGreen(color); }
QColor KmudView::getOutputBgGreen()
{ return output->getOutputBgGreen(); }

void KmudView::setOutputBgYellow(QColor color)
{ output->setOutputBgYellow(color); }
QColor KmudView::getOutputBgYellow()
{ return output->getOutputBgYellow(); }

void KmudView::setOutputBgBlue(QColor color)
{ output->setOutputBgBlue(color); }
QColor KmudView::getOutputBgBlue()
{ return output->getOutputBgBlue(); }

void KmudView::setOutputBgMagenta(QColor color)
{ output->setOutputBgMagenta(color); }
QColor KmudView::getOutputBgMagenta()
{ return output->getOutputBgMagenta(); }

void KmudView::setOutputBgCyan(QColor color)
{ output->setOutputBgCyan(color); }
QColor KmudView::getOutputBgCyan()
{ return output->getOutputBgCyan(); }

void KmudView::setOutputBgWhite(QColor color)
{ output->setOutputBgWhite(color); }
QColor KmudView::getOutputBgWhite()
{ return output->getOutputBgWhite(); }



QColor KmudView::getDebugHighlightColor()
{
    return output->getDebugHighlightColor();
}

void KmudView::setDebugHighlightColor(QColor color)
{
    output->setDebugHighlightColor(color);
}


void KmudView::setInputFont( QFont font )
{
 input->setFont(font);
 input2->setFont(font);
}

QFont KmudView::getInputFont()
{
 return input->font();
}

void KmudView::setInputBackgroundColor(QColor bgc)
{
// QLineEdit will only set the frame to bgc not the background itself,
// the real background will be set either to white (light bgc) or black (dark bgc)
// so to avoid a black frame on a black background set it to grey in this special case
// sorry, but dont know what QLineEdit is doing there %-|
 if (bgc==QColor(0,0,0))
  {
   input->setPalette(QPalette(QColor(32,32,32)));
   input2->setPalette(QPalette(QColor(32,32,32)));
  }
 else
  {
   input->setPalette(QPalette(bgc));
   input2->setPalette(QPalette(bgc));
  }
}

QColor KmudView::getInputBackgroundColor()
{
 return input->backgroundColor();
}


void KmudView::setLocalEchoColor(QColor c)
{
 localEchoColor=c;
}

QColor KmudView::getLocalEchoColor()
{
 return localEchoColor;
}



void KmudView::resizeEvent( QResizeEvent * )
{
  ///////////////////////////////////////////////////////////////////
  // refresh Geometry settings, if view resized
	

  int i=(input->fontMetrics()).height()+8;

  if (input2->isVisible())
   {
    output->setGeometry(0,0,this->width(),(this->height())-i*2);
    input->setGeometry(0,(this->height())-i*2,this->width(),i);
    input2->setGeometry(0,(this->height())-i,this->width(),i);
   }
  else
   {
    output->setGeometry(0,0,this->width(),(this->height())-i);
    input->setGeometry(0,(this->height())-i,this->width(),i);
    input2->setGeometry(0,(this->height())-i,this->width(),i);
   }


}


void KmudView::slotFlushTelnetBuffer(QString buf)
{
 output->append(buf,output->getForeground(),output->getBackground(),output->getFont());
}

void KmudView::slotMacroPressed(QString macro)
{
   AliasEntry* alias;
   CCharacterProfile* prof;
   QString s = macro;
   QString tmp1, tmp2;

   prof = document->getCharacterProfile(document->getCurrentCharacterID());
   if (prof)
   {

    /** Parse semi-colons */

    alias = prof->firstAlias();
    tmp1 = s;
    s = "";

    /** bugfix to allow the enter character by itself */
    if (tmp1 == QString("")) {
        s = "\n";
    }

    /** This code parses input, replaces ; with \n, and parses
     ** aliases into their definitions.
     **/
    if (separator != "") { // Empty disables
    while (tmp1 != "") {
        if (tmp1.contains(separator)) {
            uint pos = (uint)tmp1.find(separator, 0, false);

            tmp2 = tmp1;
            tmp2.truncate(pos);
            tmp1.remove(0, pos);
            if (tmp1.find(separator, 0, true) == 0)
                tmp1.remove(0, separator.length());

            alias = prof->firstAlias();
            while (alias != NULL) {
                if (alias->name == tmp2) {
                    tmp2 = alias->definition;
                    if (tmp2.find(separator, 0, true) >= 0) {
                        while (tmp2.find(separator, 0, true) >= 0) {
                            tmp2 = tmp2.replace((uint)tmp2.find(separator, 0, false), (uint)separator.length(), "\n");
                        }
                    }
                }
                alias = prof->nextAlias();
            }
            tmp2.append("\n");
            s.append(tmp2);
        }
        else {
            alias = prof->firstAlias();
            while (alias != NULL) {
                if (alias->name == tmp1) {
                    tmp1 = alias->definition;
                    if (tmp1.find(separator, 0, true) >= 0) {
                        while (tmp1.find(separator, 0, true) >= 0) {
                            tmp1 = tmp1.replace((uint)tmp1.find(separator, 0, false), (uint)separator.length(), "\n");
                        }
                    }
                }
                alias = prof->nextAlias();
            }

            s.append(tmp1);
            tmp1 = "";
            s.append("\n");
        }
    } // end of   while (tmp1 != "")
    } // end of   if (separator != "")
    else {
        s.append(tmp1);
        s.append("\n");
    }

   } else s.append("\n");

   if ((getLocalEchoMode()==1) || ((getLocalEcho()==true) && (getLocalEchoMode()==2))) {
       output->append(s,localEchoColor,output->getBackground(),output->getFont());
   } else {
       output->append("\n",localEchoColor,output->getBackground(),output->getFont());
   }

   emit enteredInput(s,s.size()-1);  // -1 because else receive "^@" at end of response
//   input->clear();                   // don't know why...
}

void KmudView::slotInputReturnPressed()
{
   AliasEntry* alias;
   CCharacterProfile* prof;
   QString s = input->text();
   QString tmp1, tmp2;

   prof = document->getCharacterProfile(document->getCurrentCharacterID());
   if (prof)
   {

    /** Parse semi-colons */

    alias = prof->firstAlias();
    tmp1 = s;
    s = "";

    /** bugfix to allow the enter character by itself */
    if (tmp1 == QString("")) {
        s = "\n";
    }

    /** This code parses input, replaces ; with \n, and parses
     ** aliases into their definitions.
     **/
    if (separator != "") { // Empty disables
    while (tmp1 != "") {
        if (tmp1.contains(separator)) {
            uint pos = (uint)tmp1.find(separator, 0, false);

            tmp2 = tmp1;
            tmp2.truncate(pos);
            tmp1.remove(0, pos);
            if (tmp1.find(separator, 0, true) == 0)
                tmp1.remove(0, separator.length());

            alias = prof->firstAlias();
            while (alias != NULL) {
                if (alias->name == tmp2) {
                    tmp2 = alias->definition;
                    if (tmp2.find(separator, 0, true) >= 0) {
                        while (tmp2.find(separator, 0, true) >= 0) {
                            tmp2 = tmp2.replace((uint)tmp2.find(separator, 0, false), (uint)separator.length(), "\n");
                        }
                    }
                }
                alias = prof->nextAlias();
            }
            tmp2.append("\n");
            s.append(tmp2);
        }
        else {
            alias = prof->firstAlias();
            while (alias != NULL) {
                if (alias->name == tmp1) {
                    tmp1 = alias->definition;
                    if (tmp1.find(separator, 0, true) >= 0) {
                        while (tmp1.find(separator, 0, true) >= 0) {
                            tmp1 = tmp1.replace((uint)tmp1.find(separator, 0, false), (uint)separator.length(), "\n");
                        }
                    }
                }
                alias = prof->nextAlias();
            }

            s.append(tmp1);
            tmp1 = "";
            s.append("\n");
        }
    } // end of   while (tmp1 != "")
    } // end of   if (separator != "")
    else {
        s.append(tmp1);
        s.append("\n");
    }


   } else s.append("\n");

   if ((getLocalEchoMode()==1) || ((getLocalEcho()==true) && (getLocalEchoMode()==2))) {
       output->append(s,localEchoColor,output->getBackground(),output->getFont());
   } else {
       output->append("\n",localEchoColor,output->getBackground(),output->getFont());
   }

   emit enteredInput(s,s.size()-1);  // -1 because else receive "^@" at end of response
   input->clear();                   // don't know why...
}


void KmudView::slotInput2ReturnPressed()
{
 QString s = input2->text();

 s.append("\n");
 if ((getLocalEchoMode()==1) || ((getLocalEcho()==true) && (getLocalEchoMode()==2)))
  { output->append(s,localEchoColor,output->getBackground(),output->getFont()); }
 emit enteredInput(s,s.size()-1);  // -1 because else receive "^@" at end of response
 input2->clear();                   // don't know why...

}

void KmudView::slotMudClosed()
{
 output->append(i18n("\nConnection closed by foreign host.\n"),output->getForeground(),output->getBackground(),output->getFont());
}


bool KmudView::getLocalEcho()
{
 return localEcho;
}

void KmudView::setLocalEcho(bool b)
{
 localEcho=b;
}

int KmudView::getLocalEchoMode()
{
 return localEchoMode;
}

void KmudView::setLocalEchoMode(int i)
{
 localEchoMode=i;
}

bool KmudView::getInputEcho()
{
 return inputEcho;
}

void KmudView::setInputEcho(bool b)
{
 inputEcho=b;

 if (getInputEchoMode()==0) input->setEchoMode(QLineEdit::NoEcho);
 if (getInputEchoMode()==1) input->setEchoMode(QLineEdit::Normal);
 if ((getInputEchoMode()==2) && (getInputEcho()==true)) input->setEchoMode(QLineEdit::Normal);
 if ((getInputEchoMode()==2) && (getInputEcho()==false)) input->setEchoMode(QLineEdit::NoEcho);

 if (getInputEchoMode()==0) input2->setEchoMode(QLineEdit::NoEcho);
 if (getInputEchoMode()==1) input2->setEchoMode(QLineEdit::Normal);
 if ((getInputEchoMode()==2) && (getInputEcho()==true)) input2->setEchoMode(QLineEdit::Normal);
 if ((getInputEchoMode()==2) && (getInputEcho()==false)) input2->setEchoMode(QLineEdit::NoEcho);

}

int KmudView::getInputEchoMode()
{
 return inputEchoMode;
}

void KmudView::setInputEchoMode(int i)
{

 inputEchoMode=i;

 if (getInputEchoMode()==0) input->setEchoMode(QLineEdit::NoEcho);
 if (getInputEchoMode()==1) input->setEchoMode(QLineEdit::Normal);
 if ((getInputEchoMode()==2) && (getInputEcho()==true)) input->setEchoMode(QLineEdit::Normal);
 if ((getInputEchoMode()==2) && (getInputEcho()==false)) input->setEchoMode(QLineEdit::NoEcho);

 if (getInputEchoMode()==0) input2->setEchoMode(QLineEdit::NoEcho);
 if (getInputEchoMode()==1) input2->setEchoMode(QLineEdit::Normal);
 if ((getInputEchoMode()==2) && (getInputEcho()==true)) input2->setEchoMode(QLineEdit::Normal);
 if ((getInputEchoMode()==2) && (getInputEcho()==false)) input2->setEchoMode(QLineEdit::NoEcho);

}



void KmudView::setAltInput(bool b)
{
if (b)
 {
  input2->show();
  resizeEvent(NULL);
 }
else
 {
  input2->hide();
  resizeEvent(NULL);
 }
}

void KmudView::setMaxHistoryLines(int l)
{
 input->setMaxHistoryLines(l);
 input2->setMaxHistoryLines(l);
}


void KmudView::setMaxMudViewLines(int l)
{
 output->setMaxLines(l);
}


bool KmudView::getKeepLastInput()
{
 return input->getKeepLastInput();
}

void KmudView::setKeepLastInput(bool b)
{
 input->setKeepLastInput(b);
 input2->setKeepLastInput(b);
}

void KmudView::setSeparator(QString s) {
    separator = s;
}

QString KmudView::getSeparator() {
    return separator;
}

bool KmudView::getUseBeep() {
    return output->getUseBeep();
}

void KmudView::setUseBeep(bool u) {
    output->setUseBeep(u);
}

