/***************************************************************************
                          kmudview.h
                      -------------------
    description          : View Class (Main View)
    begin                : Sun Jun 20 14:17:54 MEST 1999
    copyright            : (C) 1999 by Stephan Uhlmann
    email                : suhlmann@gmx.de                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#ifndef KMUDVIEW_H 
#define KMUDVIEW_H 

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kapp.h>
#include <qwidget.h>
#include <qfont.h>
#include <qregexp.h>
#include "cmudview.h"
#include "cmudlineedit.h"

#include "kmuddoc.h"


/**
	* This class provides the Kmud application view.
  *@author Stephan Uhlmann
	*/

class KmudView : public QWidget
{
  Q_OBJECT

 public:
  /** Constructor for the main view */
  KmudView(KApplication* a=0,KmudDoc* doc=0,QWidget *parent = 0);
  /** Destructor for the main view */
  ~KmudView();

  void setDebugHighlightColor(QColor);
  QColor getDebugHighlightColor();

  void setOutputForegroundColor(QColor);
  QColor getOutputForegroundColor();

  void setOutputBackgroundColor(QColor);
  QColor getOutputBackgroundColor();

  void setOutputFont(QFont);
  QFont getOutputFont();

  void setInputFont(QFont);
  QFont getInputFont();

  void setInputBackgroundColor(QColor);
  QColor getInputBackgroundColor();

  void setLocalEchoColor(QColor);
  QColor getLocalEchoColor();

  void setInterpretType(int);
  int getInterpretType();

  void setSeparator(QString s);
  QString getSeparator();

  void setUseBeep(bool u);
  bool getUseBeep();

  void setOutputANSIBlack(QColor);
  QColor getOutputANSIBlack();
  void setOutputANSIDarkRed(QColor);
  QColor getOutputANSIDarkRed();
  void setOutputANSIDarkGreen(QColor);
  QColor getOutputANSIDarkGreen();
  void setOutputANSIDarkYellow(QColor);
  QColor getOutputANSIDarkYellow();
  void setOutputANSIDarkBlue(QColor);
  QColor getOutputANSIDarkBlue();
  void setOutputANSIDarkMagenta(QColor);
  QColor getOutputANSIDarkMagenta();
  void setOutputANSIDarkCyan(QColor);
  QColor getOutputANSIDarkCyan();
  void setOutputANSIDarkWhite(QColor);
  QColor getOutputANSIDarkWhite();

  void setOutputANSIGray(QColor);
  QColor getOutputANSIGray();
  void setOutputANSILightRed(QColor);
  QColor getOutputANSILightRed();
  void setOutputANSILightGreen(QColor);
  QColor getOutputANSILightGreen();
  void setOutputANSILightYellow(QColor);
  QColor getOutputANSILightYellow();
  void setOutputANSILightBlue(QColor);
  QColor getOutputANSILightBlue();
  void setOutputANSILightMagenta(QColor);
  QColor getOutputANSILightMagenta();
  void setOutputANSILightCyan(QColor);
  QColor getOutputANSILightCyan();
  void setOutputANSILightWhite(QColor);
  QColor getOutputANSILightWhite();

  void setOutputBgBlack(QColor);
  QColor getOutputBgBlack();
  void setOutputBgRed(QColor);
  QColor getOutputBgRed();
  void setOutputBgGreen(QColor);
  QColor getOutputBgGreen();
  void setOutputBgYellow(QColor);
  QColor getOutputBgYellow();
  void setOutputBgBlue(QColor);
  QColor getOutputBgBlue();
  void setOutputBgMagenta(QColor);
  QColor getOutputBgMagenta();
  void setOutputBgCyan(QColor);
  QColor getOutputBgCyan();
  void setOutputBgWhite(QColor);
  QColor getOutputBgWhite();


  bool getLocalEcho();
  int getLocalEchoMode();
  bool getInputEcho();
  int getInputEchoMode();

  void setKeepLastInput(bool);
  bool getKeepLastInput();


 public slots:
  void slotInputReturnPressed();
  void slotInput2ReturnPressed();
  void slotFlushTelnetBuffer(QString);
  void setLocalEcho(bool);
  void setLocalEchoMode(int);
  void setInputEcho(bool);
  void setInputEchoMode(int);
  void setAltInput(bool);

	void setMaxHistoryLines(int);
  void setMaxMudViewLines(int);

  void slotMudClosed();
  void slotMacroPressed(QString);

 signals:
  void enteredInput(QString, int);


 protected:

  virtual void resizeEvent(QResizeEvent *);

 private:

	/** output from mud */
	CMudView* output;
	/** input line */
	CMudLineEdit* input;
	/** alternatove input line */
	CMudLineEdit* input2;
	bool localEcho;
	int localEchoMode;
  bool inputEcho;
  int inputEchoMode;
	QColor localEchoColor;

  KmudDoc* document;


    /** The separating character */
    QString separator;
};

#endif // KMUDVIEW_H 






